@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2018 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2019 Intel Corporation. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -102,7 +102,6 @@
  * Local data structures
  * ---------------------------------------
  */
-static const struct pqos_cap *m_cap = NULL;
 static const struct pqos_cpuinfo *m_cpu = NULL;
 static int m_interface = PQOS_INTER_MSR;
 /**
@@ -134,15 +133,16 @@ get_hi_cos_id(const unsigned technology,
         const int mba_req = ((technology & (1 << PQOS_CAP_TYPE_MBA)) != 0);
         unsigned num_l2_cos = 0, num_l3_cos = 0, num_mba_cos = 0, num_cos = 0;
         int ret;
+        const struct pqos_cap *cap;
 
 	ASSERT(l2_req || l3_req || mba_req);
         if (hi_class_id == NULL)
                 return PQOS_RETVAL_PARAM;
 
-        ASSERT(m_cap != NULL);
+        _pqos_cap_get(&cap, NULL);
 
         if (l3_req) {
-                ret = pqos_l3ca_get_cos_num(m_cap, &num_l3_cos);
+                ret = pqos_l3ca_get_cos_num(cap, &num_l3_cos);
                 if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
                         return ret;
 
@@ -153,7 +153,7 @@ get_hi_cos_id(const unsigned technology,
         }
 
         if (l2_req) {
-                ret = pqos_l2ca_get_cos_num(m_cap, &num_l2_cos);
+                ret = pqos_l2ca_get_cos_num(cap, &num_l2_cos);
                 if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
                         return ret;
                 if (num_l2_cos == 0)
@@ -164,7 +164,7 @@ get_hi_cos_id(const unsigned technology,
         }
 
         if (mba_req) {
-                ret = pqos_mba_get_cos_num(m_cap, &num_mba_cos);
+                ret = pqos_mba_get_cos_num(cap, &num_mba_cos);
                 if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
                         return ret;
 
@@ -290,7 +290,10 @@ pqos_alloc_init(const struct pqos_cpuinfo *cpu,
 {
         int ret = PQOS_RETVAL_OK;
 
-        m_cap = cap;
+#ifndef __linux__
+        UNUSED_PARAM(cap);
+#endif
+
         m_cpu = cpu;
         if (cfg == NULL)
                 m_interface = PQOS_INTER_MSR;
@@ -310,7 +313,6 @@ pqos_alloc_fini(void)
 {
         int ret = PQOS_RETVAL_OK;
 
-        m_cap = NULL;
         m_cpu = NULL;
 #ifdef __linux__
         if (m_interface == PQOS_INTER_OS)
@@ -333,24 +335,27 @@ hw_l3ca_set(const unsigned socket,
         int ret = PQOS_RETVAL_OK;
         unsigned i = 0, count = 0, core = 0;
         int cdp_enabled = 0;
+        const struct pqos_cap *cap;
+        const struct pqos_cpuinfo *cpu;
 
         ASSERT(ca != NULL);
         ASSERT(num_ca != 0);
-        ASSERT(m_cap != NULL);
 
-        ret = pqos_l3ca_get_cos_num(m_cap, &count);
+        _pqos_cap_get(&cap, &cpu);
+
+        ret = pqos_l3ca_get_cos_num(cap, &count);
         if (ret != PQOS_RETVAL_OK)
                 return ret;             /**< perhaps no L3CA capability */
 
         if (num_ca > count)
                 return PQOS_RETVAL_ERROR;
 
-        ret = pqos_l3ca_cdp_enabled(m_cap, NULL, &cdp_enabled);
+        ret = pqos_l3ca_cdp_enabled(cap, NULL, &cdp_enabled);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
         ASSERT(m_cpu != NULL);
-        ret = pqos_cpu_get_one_core(m_cpu, socket, &core);
+        ret = pqos_cpu_get_one_core(cpu, socket, &core);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -410,16 +415,19 @@ hw_l3ca_get(const unsigned socket,
         uint64_t val = 0;
         int retval = MACHINE_RETVAL_OK;
         int cdp_enabled = 0;
+        const struct pqos_cap *cap;
 
         ASSERT(num_ca != NULL);
         ASSERT(ca != NULL);
         ASSERT(max_num_ca != 0);
-        ASSERT(m_cap != NULL);
-        ret = pqos_l3ca_get_cos_num(m_cap, &count);
+
+        _pqos_cap_get(&cap, NULL);
+
+        ret = pqos_l3ca_get_cos_num(cap, &count);
         if (ret != PQOS_RETVAL_OK)
                 return ret;             /**< perhaps no L3CA capability */
 
-        ret = pqos_l3ca_cdp_enabled(m_cap, NULL, &cdp_enabled);
+        ret = pqos_l3ca_cdp_enabled(cap, NULL, &cdp_enabled);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -476,14 +484,13 @@ hw_l3ca_get_min_cbm_bits(unsigned *min_cbm_bits)
 	const struct pqos_capability *l3_cap = NULL;
 	struct pqos_l3ca l3ca_config[PQOS_MAX_L3CA_COS];
 
-	ASSERT(m_cap != NULL);
 	ASSERT(m_cpu != NULL);
 	ASSERT(min_cbm_bits != NULL);
 
 	/**
 	 * Get L3 CAT capabilities
 	 */
-	ret = pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L3CA, &l3_cap);
+	ret = _pqos_cap_get_type(PQOS_CAP_TYPE_L3CA, &l3_cap);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* L3 CAT not supported */
 
@@ -599,16 +606,18 @@ hw_l2ca_set(const unsigned l2id,
         int ret = PQOS_RETVAL_OK;
         unsigned i = 0, count = 0, core = 0;
         int cdp_enabled = 0;
+        const struct pqos_cap *cap;
+        const struct pqos_cpuinfo *cpu;
 
         ASSERT(ca != NULL);
         ASSERT(num_ca != 0);
-        ASSERT(m_cap != NULL);
-        ASSERT(m_cpu != NULL);
+
+        _pqos_cap_get(&cap, &cpu);
 
         /*
          * Check if L2 CAT is supported
          */
-        ret = pqos_l2ca_get_cos_num(m_cap, &count);
+        ret = pqos_l2ca_get_cos_num(cap, &count);
         if (ret != PQOS_RETVAL_OK)
                 return PQOS_RETVAL_RESOURCE; /* L2 CAT not supported */
 
@@ -623,7 +632,7 @@ hw_l2ca_set(const unsigned l2id,
                 }
         }
 
-        ret = pqos_l2ca_cdp_enabled(m_cap, NULL, &cdp_enabled);
+        ret = pqos_l2ca_cdp_enabled(cap, NULL, &cdp_enabled);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -631,7 +640,7 @@ hw_l2ca_set(const unsigned l2id,
          * Pick one core from the L2 cluster and
          * perform MSR writes to COS registers on the cluster.
          */
-        ret = pqos_cpu_get_one_by_l2id(m_cpu, l2id, &core);
+        ret = pqos_cpu_get_one_by_l2id(cpu, l2id, &core);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -688,16 +697,19 @@ hw_l2ca_get(const unsigned l2id,
         unsigned i = 0, count = 0;
         unsigned core = 0;
         int cdp_enabled = 0;
+        const struct pqos_cap *cap;
 
 	ASSERT(num_ca != NULL);
 	ASSERT(ca != NULL);
 	ASSERT(max_num_ca != 0);
-	ASSERT(m_cap != NULL);
-	ret = pqos_l2ca_get_cos_num(m_cap, &count);
+
+        _pqos_cap_get(&cap, NULL);
+
+	ret = pqos_l2ca_get_cos_num(cap, &count);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* L2 CAT not supported */
 
-        ret = pqos_l2ca_cdp_enabled(m_cap, NULL, &cdp_enabled);
+        ret = pqos_l2ca_cdp_enabled(cap, NULL, &cdp_enabled);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -757,14 +769,13 @@ hw_l2ca_get_min_cbm_bits(unsigned *min_cbm_bits)
 	const struct pqos_capability *l2_cap = NULL;
 	struct pqos_l2ca l2ca_config[PQOS_MAX_L2CA_COS];
 
-	ASSERT(m_cap != NULL);
 	ASSERT(m_cpu != NULL);
 	ASSERT(min_cbm_bits != NULL);
 
 	/**
 	 * Get L2 CAT capabilities
 	 */
-	ret = pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L2CA, &l2_cap);
+	ret = _pqos_cap_get_type(PQOS_CAP_TYPE_L2CA, &l2_cap);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* L2 CAT not supported */
 
@@ -886,8 +897,7 @@ hw_mba_set(const unsigned socket,
         /**
          * Check if MBA is supported
          */
-        ASSERT(m_cap != NULL);
-        ret = pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_MBA, &mba_cap);
+        ret = _pqos_cap_get_type(PQOS_CAP_TYPE_MBA, &mba_cap);
         if (ret != PQOS_RETVAL_OK)
                 return PQOS_RETVAL_RESOURCE; /* MBA not supported */
 
@@ -902,15 +912,22 @@ hw_mba_set(const unsigned socket,
                 return PQOS_RETVAL_RESOURCE;
         }
         /**
-         * Check if class id's are within allowed range.
+         * Check if class id's are within allowed range
+	 * and if a controller is not requested.
          */
-        for (i = 0; i < num_cos; i++)
+        for (i = 0; i < num_cos; i++) {
                 if (requested[i].class_id >= count) {
                         LOG_ERROR("MBA COS%u is out of range (COS%u is max)!\n",
                                   requested[i].class_id, count - 1);
                         return PQOS_RETVAL_PARAM;
                 }
 
+                if (requested[i].ctrl != 0) {
+                        LOG_ERROR("MBA controller not supported!\n");
+                        return PQOS_RETVAL_PARAM;
+                }
+        }
+
         ASSERT(m_cpu != NULL);
         ret = pqos_cpu_get_one_core(m_cpu, socket, &core);
         if (ret != PQOS_RETVAL_OK)
@@ -920,7 +937,7 @@ hw_mba_set(const unsigned socket,
                 const uint32_t reg =
                         requested[i].class_id + PQOS_MSR_MBA_MASK_START;
                 uint64_t val = PQOS_MBA_LINEAR_MAX -
-                        (((requested[i].mb_rate + (step/2)) / step) * step);
+                        (((requested[i].mb_max + (step/2)) / step) * step);
                 int retval = MACHINE_RETVAL_OK;
 
                 if (val > mba_cap->u.mba->throttle_max)
@@ -942,7 +959,7 @@ hw_mba_set(const unsigned socket,
                         return PQOS_RETVAL_ERROR;
 
                 actual[i] = requested[i];
-                actual[i].mb_rate = (PQOS_MBA_LINEAR_MAX - val);
+                actual[i].mb_max = (PQOS_MBA_LINEAR_MAX - val);
         }
 
         return ret;
@@ -956,12 +973,15 @@ hw_mba_get(const unsigned socket,
 {
         int ret = PQOS_RETVAL_OK;
         unsigned i = 0, count = 0, core = 0;
+        const struct pqos_cap *cap;
 
         ASSERT(num_cos != NULL);
 	ASSERT(mba_tab != NULL);
 	ASSERT(max_num_cos != 0);
-        ASSERT(m_cap != NULL);
-        ret = pqos_mba_get_cos_num(m_cap, &count);
+
+        _pqos_cap_get(&cap, NULL);
+
+        ret = pqos_mba_get_cos_num(cap, &count);
         if (ret != PQOS_RETVAL_OK)
                 return ret;             /**< no MBA capability */
 
@@ -981,8 +1001,9 @@ hw_mba_get(const unsigned socket,
                 if (retval != MACHINE_RETVAL_OK)
                         return PQOS_RETVAL_ERROR;
 
-		mba_tab[i].class_id = i;
-                mba_tab[i].mb_rate = (unsigned) PQOS_MBA_LINEAR_MAX - val;
+                mba_tab[i].ctrl = 0;
+                mba_tab[i].class_id = i;
+                mba_tab[i].mb_max = (unsigned) PQOS_MBA_LINEAR_MAX - val;
         }
         *num_cos = count;
 
@@ -1024,18 +1045,20 @@ hw_alloc_assoc_set(const unsigned lcore,
 {
         int ret = PQOS_RETVAL_OK;
         unsigned num_l2_cos = 0, num_l3_cos = 0;
+        const struct pqos_cap *cap;
 
         ASSERT(m_cpu != NULL);
         ret = pqos_cpu_check_core(m_cpu, lcore);
         if (ret != PQOS_RETVAL_OK)
                 return PQOS_RETVAL_PARAM;
 
-        ASSERT(m_cap != NULL);
-        ret = pqos_l3ca_get_cos_num(m_cap, &num_l3_cos);
+        _pqos_cap_get(&cap, NULL);
+
+        ret = pqos_l3ca_get_cos_num(cap, &num_l3_cos);
         if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
                 return ret;
 
-        ret = pqos_l2ca_get_cos_num(m_cap, &num_l2_cos);
+        ret = pqos_l2ca_get_cos_num(cap, &num_l2_cos);
         if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
                 return ret;
 
@@ -1054,6 +1077,7 @@ hw_alloc_assoc_get(const unsigned lcore,
 {
         const struct pqos_capability *l3_cap = NULL;
         const struct pqos_capability *l2_cap = NULL;
+        const struct pqos_capability *mba_cap = NULL;
         int ret = PQOS_RETVAL_OK;
 
         ASSERT(class_id != NULL);
@@ -1062,17 +1086,20 @@ hw_alloc_assoc_get(const unsigned lcore,
         if (ret != PQOS_RETVAL_OK)
                 return PQOS_RETVAL_PARAM;
 
-        ASSERT(m_cap != NULL);
-        ret = pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L3CA, &l3_cap);
+        ret = _pqos_cap_get_type(PQOS_CAP_TYPE_L3CA, &l3_cap);
         if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
                 return ret;
 
-        ret = pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L2CA, &l2_cap);
+        ret = _pqos_cap_get_type(PQOS_CAP_TYPE_L2CA, &l2_cap);
         if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
                 return ret;
 
-        if (l2_cap == NULL && l3_cap == NULL)
-                /* no L2/L3 CAT detected */
+        ret = _pqos_cap_get_type(PQOS_CAP_TYPE_MBA, &mba_cap);
+        if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
+                return ret;
+
+        if (l2_cap == NULL && l3_cap == NULL && mba_cap == NULL)
+                /* no L2/L3 CAT or MBA detected */
                 return PQOS_RETVAL_RESOURCE;
 
         ret = cos_assoc_get(lcore, class_id);
@@ -1316,12 +1343,14 @@ alloc_assoc_reset(void)
 
 int
 hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
-               const enum pqos_cdp_config l2_cdp_cfg)
+               const enum pqos_cdp_config l2_cdp_cfg,
+               const enum pqos_mba_config mba_cfg)
 {
         unsigned *sockets = NULL;
         unsigned sockets_num = 0;
         unsigned *l2ids = NULL;
         unsigned l2id_num = 0;
+        const struct pqos_cap *cap;
         const struct pqos_capability *alloc_cap = NULL;
         const struct pqos_cap_l3ca *l3_cap = NULL;
         const struct pqos_cap_l2ca *l2_cap = NULL;
@@ -1340,20 +1369,26 @@ hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
                l2_cdp_cfg == PQOS_REQUIRE_CDP_OFF ||
                l2_cdp_cfg == PQOS_REQUIRE_CDP_ANY);
 
+        ASSERT(mba_cfg == PQOS_MBA_DEFAULT ||
+               mba_cfg == PQOS_MBA_CTRL ||
+               mba_cfg == PQOS_MBA_ANY);
+
+        _pqos_cap_get(&cap, NULL);
+
         /* Get L3 CAT capabilities */
-        (void) pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L3CA, &alloc_cap);
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_L3CA, &alloc_cap);
         if (alloc_cap != NULL)
                 l3_cap = alloc_cap->u.l3ca;
 
         /* Get L2 CAT capabilities */
         alloc_cap = NULL;
-        (void) pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L2CA, &alloc_cap);
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_L2CA, &alloc_cap);
         if (alloc_cap != NULL)
                 l2_cap = alloc_cap->u.l2ca;
 
         /* Get MBA capabilities */
         alloc_cap = NULL;
-        (void) pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_MBA, &alloc_cap);
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_MBA, &alloc_cap);
         if (alloc_cap != NULL)
                 mba_cap = alloc_cap->u.mba;
 
@@ -1375,8 +1410,15 @@ hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
                 ret = PQOS_RETVAL_RESOURCE;
                 goto pqos_alloc_reset_exit;
         }
+        /* Check MBA CTRL requested while not present */
+        if (mba_cap == NULL && mba_cfg != PQOS_MBA_ANY) {
+                LOG_ERROR("MBA CTRL setting requested"
+                          " but no MBA CTRL present!\n");
+                ret = PQOS_RETVAL_RESOURCE;
+                goto pqos_alloc_reset_exit;
+        }
         if (l3_cap != NULL) {
-                ret = pqos_l3ca_cdp_enabled(m_cap, &cdp_supported, NULL);
+                ret = pqos_l3ca_cdp_enabled(cap, &cdp_supported, NULL);
                 if (ret != PQOS_RETVAL_OK)
                         goto pqos_alloc_reset_exit;
 
@@ -1394,7 +1436,7 @@ hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
                         max_l3_cos = max_l3_cos * 2;
         }
         if (l2_cap != NULL) {
-                ret = pqos_l2ca_cdp_enabled(m_cap, &cdp_supported, NULL);
+                ret = pqos_l2ca_cdp_enabled(cap, &cdp_supported, NULL);
                 if (ret != PQOS_RETVAL_OK)
                         goto pqos_alloc_reset_exit;
 
@@ -1411,6 +1453,12 @@ hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
                 if (l2_cap->cdp && l2_cap->cdp_on)
                         max_l2_cos = max_l2_cos * 2;
         }
+        if (mba_cap != NULL && mba_cfg == PQOS_MBA_CTRL) {
+                LOG_ERROR("MBA CTRL requested but not supported by the "
+                          "platform!\n");
+                ret = PQOS_RETVAL_PARAM;
+                goto pqos_alloc_reset_exit;
+        }
 
         /**
          * Get number & list of sockets in the system
