@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2018 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2019 Intel Corporation. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -138,8 +138,6 @@
  * Local data structures
  * ---------------------------------------
  */
-static const struct pqos_cap *m_cap = NULL; /**< capabilities structure
-                                               passed from cap */
 static const struct pqos_cpuinfo *m_cpu = NULL; /**< cpu topology passed
                                                    from cap */
 static unsigned m_rmid_max = 0;         /**< max RMID */
@@ -216,7 +214,7 @@ pqos_mon_init(const struct pqos_cpuinfo *cpu,
 
         ASSERT(item != NULL);
         m_rmid_max = item->u.mon->max_rmid;
-        if (m_rmid_max == 0) {
+        if (cfg->interface == PQOS_INTER_MSR && m_rmid_max == 0) {
                 pqos_mon_fini();
                 return PQOS_RETVAL_PARAM;
         }
@@ -231,7 +229,6 @@ pqos_mon_init(const struct pqos_cpuinfo *cpu,
 #endif
  pqos_mon_init_exit:
         m_cpu = cpu;
-        m_cap = cap;
 #ifdef __linux__
         m_interface = cfg->interface;
 #else
@@ -252,7 +249,6 @@ pqos_mon_fini(void)
                 ret = os_mon_fini();
 #endif
         m_cpu = NULL;
-        m_cap = NULL;
 
         return ret;
 }
@@ -302,8 +298,7 @@ rmid_alloc(const unsigned cluster,
          * - look for the \a event in the event list
          * - find max RMID matching the \a event
          */
-        ASSERT(m_cap != NULL);
-        ret = pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_MON, &item);
+        ret = _pqos_cap_get_type(PQOS_CAP_TYPE_MON, &item);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
         ASSERT(item != NULL);
@@ -390,10 +385,13 @@ rmid_alloc(const unsigned cluster,
 static uint64_t
 scale_event(const enum pqos_mon_event event, const uint64_t val)
 {
+        const struct pqos_cap *cap;
         const struct pqos_monitor *pmon;
         int ret;
 
-        ret = pqos_cap_get_event(m_cap, event, &pmon);
+        _pqos_cap_get(&cap, NULL);
+
+        ret = pqos_cap_get_event(cap, event, &pmon);
         ASSERT(ret == PQOS_RETVAL_OK);
         if (ret != PQOS_RETVAL_OK)
                 return val;
@@ -892,6 +890,7 @@ hw_mon_start(const unsigned num_cores,
         unsigned i = 0;
         int ret = PQOS_RETVAL_OK;
         int retval = PQOS_RETVAL_OK;
+        const struct pqos_cap *cap;
 
         ASSERT(group != NULL);
         ASSERT(cores != NULL);
@@ -901,18 +900,20 @@ hw_mon_start(const unsigned num_cores,
 
         memset(ctxs, 0, sizeof(ctxs));
 
+        _pqos_cap_get(&cap, NULL);
+
         /**
          * Validate if event is listed in capabilities
          */
         for (i = 0; i < (sizeof(event) * 8); i++) {
                 const enum pqos_mon_event evt_mask =
-                        (enum pqos_mon_event)(1 << i);
+                        (enum pqos_mon_event)(1U << i);
                 const struct pqos_monitor *ptr = NULL;
 
                 if (!(evt_mask & event))
                         continue;
 
-                ret = pqos_cap_get_event(m_cap, evt_mask, &ptr);
+                ret = pqos_cap_get_event(cap, evt_mask, &ptr);
                 if (ret != PQOS_RETVAL_OK || ptr == NULL)
                         return PQOS_RETVAL_PARAM;
         }
@@ -1126,14 +1127,14 @@ int
 hw_mon_poll(struct pqos_mon_data **groups,
               const unsigned num_groups)
 {
-        int ret = PQOS_RETVAL_OK;
         unsigned i = 0;
 
         ASSERT(groups != NULL);
         ASSERT(num_groups > 0);
 
         for (i = 0; i < num_groups; i++) {
-                ret = pqos_core_poll(groups[i]);
+                int ret = pqos_core_poll(groups[i]);
+
                 if (ret != PQOS_RETVAL_OK)
                         LOG_WARN("Failed to read event on "
                                  "core %u\n", groups[i]->cores[0]);
@@ -1195,3 +1196,22 @@ get_delta(const uint64_t old_value, const uint64_t new_value)
         else
                 return new_value - old_value;
 }
+
+int
+pqos_mon_tids_get(const struct pqos_mon_data *group,
+                  pid_t *tids,
+                  unsigned max_num_tids,
+                  unsigned *num_tids)
+{
+        unsigned i = 0;
+
+        if (group == NULL || tids == NULL || num_tids == NULL)
+                return PQOS_RETVAL_PARAM;
+
+        for (i = 0; i < group->tid_nr && i < max_num_tids; i++)
+                tids[i] = group->tid_map[i];
+
+        *num_tids = i;
+
+        return PQOS_RETVAL_OK;
+}
