@@ -37,6 +37,7 @@
 
 #include "pqos.h"
 
+#include "common.h"
 #include "os_cap.h"
 #include "types.h"
 #include "log.h"
@@ -53,15 +54,18 @@ static int mba_ctrl = -1; /**< mba ctrl support */
 /**
  * @brief Checks file fname to detect str and sets a flag
  *
- * @param fname name of the file to be searched
- * @param str string being searched for
- * @param supported pointer to os_supported flag
+ * @param [in] fname name of the file to be searched
+ * @param [in] str string being searched for
+ * @param [in] check_symlink a flag indicating if a path must be checked
+ *             against symlinks
+ * @param [out] supported pointer to os_supported flag
  *
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
 static int
-detect_os_support(const char *fname, const char *str, int *supported)
+detect_os_support(const char *fname, const char *str, int check_symlink,
+                  int *supported)
 {
         FILE *fd;
         char temp[1024];
@@ -69,7 +73,11 @@ detect_os_support(const char *fname, const char *str, int *supported)
         if (fname == NULL || str == NULL || supported == NULL)
                 return PQOS_RETVAL_PARAM;
 
-        fd = fopen(fname, "r");
+        if (check_symlink)
+                fd = fopen_check_symlink(fname, "r");
+        else
+                fd = fopen(fname, "r");
+
         if (fd == NULL) {
                 LOG_DEBUG("%s not found.\n", fname);
                 *supported = 0;
@@ -110,7 +118,7 @@ readuint64(const char *fname, unsigned base, uint64_t *value)
         ASSERT(fname != NULL);
         ASSERT(value != NULL);
 
-        fd = fopen(fname, "r");
+        fd = fopen_check_symlink(fname, "r");
         if (fd == NULL)
                 return PQOS_RETVAL_ERROR;
 
@@ -226,7 +234,7 @@ os_cap_init(const enum pqos_interface inter)
         /**
          * resctrl detection
          */
-        ret = detect_os_support(PROC_FILESYSTEMS, "resctrl", &res_flag);
+        ret = detect_os_support(PROC_FILESYSTEMS, "resctrl", 0, &res_flag);
         if (ret != PQOS_RETVAL_OK) {
                 LOG_ERROR("Fatal error encountered in resctrl detection!\n");
                 return ret;
@@ -252,8 +260,18 @@ os_cap_init(const enum pqos_interface inter)
                 ret = resctrl_mount(PQOS_REQUIRE_CDP_OFF, PQOS_REQUIRE_CDP_OFF,
                                     PQOS_MBA_CTRL);
                 if (ret == PQOS_RETVAL_OK) {
+                        FILE *fd;
+
+                        /* Verify possibility of setting mba value above 100 */
+                        fd = resctrl_alloc_fopen(0, "schemata", "w");
+                        if (fd != NULL) {
+                                fprintf(fd, "MB:0=200\n");
+                                if (fclose(fd) == 0)
+                                        mba_ctrl = 1;
+                                else
+                                        mba_ctrl = 0;
+                        }
                         resctrl_umount();
-                        mba_ctrl = 1;
                 } else
                         mba_ctrl = 0;
 
@@ -317,16 +335,17 @@ detect_mon_resctrl_support(const enum pqos_mon_event event,
         }
 
         ret = detect_os_support(RESCTRL_PATH_INFO_L3_MON"/mon_features",
-                                 event_name, supported);
+                                 event_name, 1, supported);
 
-        *scale = 1;
+        if (scale != NULL)
+                *scale = 1;
 
         return ret;
 }
 
 
 /**
- * @brief Reads scale factor of perf monitioring event
+ * @brief Reads scale factor of perf monitoring event
  *
  * @param [in] event_name perf monitoring event name
  * @param [out] scale scale factor
@@ -351,7 +370,7 @@ get_mon_perf_scale_factor(const char *event_name, uint32_t *scale)
         snprintf(path, sizeof(path) - 1, PERF_MON_PATH"/events/%s.scale",
                  event_name);
 
-        fd = fopen(path, "r");
+        fd = fopen_check_symlink(path, "r");
         if (fd == NULL) {
                 LOG_ERROR("Failed to open %s perf monitoring event scale "
                           "file!\n", event_name);
@@ -369,7 +388,7 @@ get_mon_perf_scale_factor(const char *event_name, uint32_t *scale)
         snprintf(path, sizeof(path) - 1, PERF_MON_PATH"/events/%s.unit",
                  event_name);
 
-        fd = fopen(path, "r");
+        fd = fopen_check_symlink(path, "r");
         if (fd == NULL) {
                 LOG_ERROR("Failed to open %s perf monitoring event unit "
                           "file!\n", event_name);
@@ -479,8 +498,8 @@ detect_mon_perf_support(const enum pqos_mon_event event,
  */
 static int
 detect_mon_support(const enum pqos_mon_event event,
-                        int *supported,
-                        uint32_t *scale)
+                   int *supported,
+                   uint32_t *scale)
 {
         int ret;
 
@@ -539,7 +558,7 @@ os_cap_mon_discover(struct pqos_cap_mon **r_cap,
                 PQOS_PERF_EVENT_IPC
         };
 
-        ret = detect_os_support(PROC_CPUINFO, "cqm", &supported);
+        ret = detect_os_support(PROC_CPUINFO, "cqm", 0, &supported);
         if (ret != PQOS_RETVAL_OK) {
                 LOG_ERROR("Fatal error encountered in"
                           " OS detection!\n");
@@ -644,7 +663,7 @@ os_cap_l3ca_discover(struct pqos_cap_l3ca **r_cap,
                 goto os_cap_l3ca_discover_exit;
 
         if (!cdp_on)
-                ret = detect_os_support(PROC_CPUINFO, "cdp_l3", &cap->cdp);
+                ret = detect_os_support(PROC_CPUINFO, "cdp_l3", 0, &cap->cdp);
 
  os_cap_l3ca_discover_exit:
         if (ret == PQOS_RETVAL_OK)
@@ -698,7 +717,7 @@ os_cap_l2ca_discover(struct pqos_cap_l2ca **r_cap,
                 goto os_cap_l2ca_discover_exit;
 
         if (!cdp_on)
-                ret = detect_os_support(PROC_CPUINFO, "cdp_l2", &cap->cdp);
+                ret = detect_os_support(PROC_CPUINFO, "cdp_l2", 0, &cap->cdp);
 
  os_cap_l2ca_discover_exit:
         if (ret == PQOS_RETVAL_OK)
@@ -721,9 +740,10 @@ os_cap_get_mba_ctrl(const struct pqos_cap *cap,
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
-        if (mba_ctrl == 0) {
+        /* resctrl is mounted with default options */
+        if (mba_ctrl != -1) {
                 *enabled = 0;
-                *supported = 0;
+                *supported = mba_ctrl;
                 return PQOS_RETVAL_OK;
         }
 
@@ -732,7 +752,7 @@ os_cap_get_mba_ctrl(const struct pqos_cap *cap,
 
         /* check mount flags */
         if (*enabled == -1) {
-                ret = detect_os_support(PROC_MOUNTS, "mba_MBps", enabled);
+                ret = detect_os_support(PROC_MOUNTS, "mba_MBps", 0, enabled);
                 if (ret != PQOS_RETVAL_OK)
                         return ret;
         }
@@ -742,35 +762,49 @@ os_cap_get_mba_ctrl(const struct pqos_cap *cap,
                 unsigned grp;
                 unsigned count = 0;
                 unsigned i;
+                unsigned *sock, sock_num;
+                struct resctrl_schemata *schmt;
 
                 ret = resctrl_alloc_get_grps_num(cap, &count);
                 if (ret != PQOS_RETVAL_OK)
                         return ret;
 
-                for (grp = 0; grp < count && *enabled == -1; grp++) {
-                        struct resctrl_alloc_schemata schmt;
+                sock = pqos_cpu_get_sockets(cpu, &sock_num);
+                if (sock == NULL)
+                        return PQOS_RETVAL_ERROR;
 
-                        ret = resctrl_alloc_schemata_init(grp, cap, cpu,
-                                                          &schmt);
-                        if (ret == PQOS_RETVAL_OK)
-                                ret = resctrl_alloc_schemata_read(grp, &schmt);
-
-                        if (ret == PQOS_RETVAL_OK)
-                                for (i = 0; i < schmt.mba_num; i++)
-                                        if (schmt.mba[i].mb_max > 100) {
-                                                *enabled = 1;
-                                                break;
-                                        }
-
-                        resctrl_alloc_schemata_fini(&schmt);
+                schmt = resctrl_schemata_alloc(cap, cpu);
+                if (schmt == NULL) {
+                        free(sock);
+                        return PQOS_RETVAL_ERROR;
                 }
+
+                for (grp = 0; grp < count && *enabled == -1; grp++) {
+                        ret = resctrl_alloc_schemata_read(grp, schmt);
+                        if (ret != PQOS_RETVAL_OK)
+                                continue;
+
+                        for (i = 0; i < sock_num; i++) {
+                                struct pqos_mba mba;
+
+                                ret = resctrl_schemata_mba_get(schmt, sock[i],
+                                                               &mba);
+                                if (ret == PQOS_RETVAL_OK && mba.mb_max > 100) {
+                                        *enabled = 1;
+                                        break;
+                                }
+                        }
+                }
+
+                resctrl_schemata_free(schmt);
+                free(sock);
         }
 
         /* get free COS and try to write value above 100 */
         if (*enabled == -1) {
                 unsigned grp;
                 unsigned count = 0;
-                struct resctrl_alloc_schemata schmt;
+                struct resctrl_schemata *schmt;
                 FILE *fd;
 
                 ret = resctrl_alloc_get_grps_num(cap, &count);
@@ -784,15 +818,15 @@ os_cap_get_mba_ctrl(const struct pqos_cap *cap,
                         goto ctrl_support;
                 }
 
-                ret = resctrl_alloc_schemata_init(grp, cap, cpu, &schmt);
-                if (ret != PQOS_RETVAL_OK)
+                schmt = resctrl_schemata_alloc(cap, cpu);
+                if (schmt == NULL)
                         goto ctrl_support;
 
-                ret = resctrl_alloc_schemata_read(grp, &schmt);
+                ret = resctrl_alloc_schemata_read(grp, schmt);
                 if (ret == PQOS_RETVAL_OK) {
                         fd = resctrl_alloc_fopen(grp, "schemata", "w");
                         if (fd != NULL) {
-                                fprintf(fd, "MB:0=%u", (uint32_t)UINT32_MAX);
+                                fprintf(fd, "MB:0=2000\n");
                                 if (fclose(fd) == 0)
                                         *enabled = 1;
                                 else
@@ -800,14 +834,14 @@ os_cap_get_mba_ctrl(const struct pqos_cap *cap,
                         }
                         /* restore MBA configuration */
                         if (*enabled == 1) {
-                                ret = resctrl_alloc_schemata_write(grp, &schmt);
+                                ret = resctrl_alloc_schemata_write(grp, schmt);
                                 if (ret != PQOS_RETVAL_OK)
                                         LOG_WARN("Unable to restore MBA "
                                                  "settings\n");
                         }
                 }
 
-                resctrl_alloc_schemata_fini(&schmt);
+                resctrl_schemata_free(schmt);
         }
 
  ctrl_support:
@@ -816,27 +850,23 @@ os_cap_get_mba_ctrl(const struct pqos_cap *cap,
 
         if (*enabled == 1)
                 *supported = 1;
-        else if (mba_ctrl != -1)
-                *supported = mba_ctrl;
-        /* Check if it is possible to mount resctrl with mba_MBps enabled */
-        else if (access(RESCTRL_PATH"/cpus", F_OK) != 0) {
-                ret = resctrl_mount(PQOS_REQUIRE_CDP_OFF, PQOS_REQUIRE_CDP_OFF,
-                                    PQOS_MBA_CTRL);
-                if (ret == PQOS_RETVAL_OK)
-                        *supported = 1;
-                else {
-                        ret = resctrl_mount(PQOS_REQUIRE_CDP_OFF,
-                                            PQOS_REQUIRE_CDP_OFF,
-                                            PQOS_MBA_DEFAULT);
-                        if (ret == PQOS_RETVAL_OK)
-                                *supported = 0;
-                }
 
-                if (ret == PQOS_RETVAL_OK)
-                        resctrl_umount();
+        /* Check if MBL monitoring is supported */
+        else {
+                int mbl = 0;
+
+                ret = detect_mon_resctrl_support(PQOS_MON_EVENT_LMEM_BW, &mbl,
+                                                 NULL);
+                if (ret != PQOS_RETVAL_OK)
+                        return ret;
+                if (!mbl)
+                        *supported = 0;
         }
 
  ctrl_exit:
+        if (*supported == 0)
+                *enabled = 0;
+
         if (*supported == 1)
                 LOG_INFO("OS support for MBA CTRL detected\n");
         else if (*supported == 0)
@@ -876,7 +906,7 @@ os_cap_mba_discover(struct pqos_cap_mba **r_cap,
                 goto os_cap_mba_discover_exit;
 
         /* Detect MBA CTRL status */
-        ret = detect_os_support(PROC_MOUNTS, "mba_MBps", &(cap->ctrl_on));
+        ret = detect_os_support(PROC_MOUNTS, "mba_MBps", 0, &(cap->ctrl_on));
         if (ret != PQOS_RETVAL_OK)
                 goto os_cap_mba_discover_exit;
         if (cap->ctrl_on == 1)
@@ -910,4 +940,3 @@ os_cap_mba_discover(struct pqos_cap_mba **r_cap,
 
         return ret;
 }
-
