@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2018 Intel Corporation. All rights reserved.
+ * Copyright(c) 2018-2019 Intel Corporation. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,7 @@
 #include <linux/perf_event.h>
 
 #include "pqos.h"
+#include "common.h"
 #include "perf.h"
 #include "log.h"
 #include "types.h"
@@ -60,7 +61,6 @@
  * Local data structures
  * ---------------------------------------
  */
-static const struct pqos_cap *m_cap = NULL;
 static const struct pqos_cpuinfo *m_cpu = NULL;
 
 /**
@@ -195,7 +195,7 @@ set_mon_type(void)
         char file[64], evt[8];
 
         snprintf(file, sizeof(file) - 1, "%s%s", PERF_MON_PATH, perf_type);
-        fd = fopen(file, "r");
+        fd = fopen_check_symlink(file, "r");
         if (fd == NULL) {
                 LOG_INFO("Perf monitoring not supported. "
                          "Kernel version 4.6 or higher required.\n");
@@ -287,7 +287,7 @@ set_rdt_event_attrs(const int idx, const char *fname)
          */
         snprintf(file, sizeof(file) - 1, "%s%s%s", PERF_MON_PATH, perf_events,
                  fname);
-        fd = fopen(file, "r");
+        fd = fopen_check_symlink(file, "r");
         if (fd == NULL) {
                 LOG_ERROR("Failed to open %s!\n", file);
                 return PQOS_RETVAL_ERROR;
@@ -312,7 +312,7 @@ set_rdt_event_attrs(const int idx, const char *fname)
          */
         snprintf(file, sizeof(file) - 1, "%s%s%s.scale",
                  PERF_MON_PATH, perf_events, fname);
-        fd = fopen(file, "r");
+        fd = fopen_check_symlink(file, "r");
         if (fd == NULL) {
                 LOG_ERROR("Failed to open OS monitoring event scale file!\n");
                 return PQOS_RETVAL_ERROR;
@@ -410,54 +410,14 @@ set_mon_events(void)
         return ret;
 }
 
-/**
- * @brief Update monitoring capability structure with supported events
- *
- * @param cap pqos capability structure
- *
- * @return Operational Status
- * @retval PQOS_RETVAL_OK on success
- */
-static void
-set_mon_caps(const struct pqos_cap *cap)
-{
-        int ret;
-        unsigned i;
-        const struct pqos_capability *p_cap = NULL;
-
-        ASSERT(cap != NULL);
-
-        /* find monitoring capability */
-        ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_MON, &p_cap);
-        if (ret != PQOS_RETVAL_OK)
-                return;
-
-        /* update capabilities structure */
-        for (i = 0; i < DIM(events_tab); i++) {
-                unsigned j;
-
-                if (!events_tab[i].supported)
-                        continue;
-
-                for (j = 0; j < p_cap->u.mon->num_events; j++) {
-                        struct pqos_monitor *mon = &p_cap->u.mon->events[j];
-
-                        if (events_tab[i].event != mon->type)
-                                continue;
-                        mon->os_support = PQOS_OS_MON_PERF;
-                        LOG_INFO("Detected perf monitoring support"
-                                 " for %s\n", events_tab[j].desc);
-                        break;
-                }
-        }
-}
-
 int perf_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
 {
         int ret;
+        unsigned i;
 
         ASSERT(cpu != NULL);
-        ASSERT(cap != NULL);
+
+        UNUSED_PARAM(cap);
 
         ret = set_arch_event_attrs(&all_evt_mask);
         if (ret != PQOS_RETVAL_OK)
@@ -474,10 +434,14 @@ int perf_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
                 return ret;
 
  perf_mon_init_exit:
-        /* Update capabilities structure with perf supported events */
-        set_mon_caps(cap);
+        for (i = 0; i < DIM(events_tab); i++) {
+                if (!events_tab[i].supported)
+                        continue;
+
+                LOG_INFO("Detected perf monitoring support for %s\n",
+                         events_tab[i].desc);
+        }
 
-        m_cap = cap;
         m_cpu = cpu;
 
         return ret;
@@ -486,7 +450,6 @@ int perf_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
 int
 perf_mon_fini(void)
 {
-        m_cap = NULL;
         m_cpu = NULL;
 
         return PQOS_RETVAL_OK;
