@@ -57,7 +57,7 @@ extern "C" {
  * =======================================
  */
 
-#define PQOS_VERSION       30000        /**< version 3.0.0 */
+#define PQOS_VERSION       30100        /**< version 3.1.0 */
 #define PQOS_MAX_L3CA_COS  16           /**< 16 x COS */
 #define PQOS_MAX_L2CA_COS  16           /**< 16 x COS */
 
@@ -1242,6 +1242,22 @@ pqos_mon_get_event_value(void * const value,
 	return PQOS_RETVAL_OK;
 }
 
+/**
+ * @brief Returns all TIDs associated with the specified monitoring group.
+ *
+ * @param [in] group a monitoring group
+ * @param [out] tids an array to store TIDs
+ * @param [in] max_num_tids a maximum number of TIDs to be filled in tids array
+ * @param [out] num_tids a number of returned TIDs
+ * @return Operation status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int
+pqos_mon_tids_get(const struct pqos_mon_data *group,
+                  pid_t *tids,
+                  unsigned max_num_tids,
+                  unsigned *num_tids);
+
 #ifdef __cplusplus
 }
 #endif
