@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2018 Intel Corporation. All rights reserved.
+ * Copyright(c) 2018-2019 Intel Corporation. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,18 +44,17 @@
 #include "pqos.h"
 #include "log.h"
 #include "types.h"
+#include "cap.h"
+#include "common.h"
 #include "resctrl.h"
 #include "resctrl_monitoring.h"
 #include "resctrl_alloc.h"
 
-#define RESCTRL_PATH_INFO_L3_MON RESCTRL_PATH_INFO"/L3_MON"
-
 /**
  * ---------------------------------------
  * Local data structures
  * ---------------------------------------
  */
-static const struct pqos_cap *m_cap = NULL;
 static const struct pqos_cpuinfo *m_cpu = NULL;
 
 static int supported_events = 0;
@@ -80,37 +79,6 @@ filter(const struct dirent *dir)
         return (dir->d_name[0] == '.') ? 0 : 1;
 }
 
-/**
- * @brief Update monitoring capability structure with supported events
- *
- * @param cap pqos capability structure
- *
- * @return Operational Status
- * @retval PQOS_RETVAL_OK on success
- */
-static void
-set_mon_caps(const struct pqos_cap *cap)
-{
-        int ret;
-        unsigned i;
-        const struct pqos_capability *p_cap = NULL;
-
-        ASSERT(cap != NULL);
-
-        /* find monitoring capability */
-        ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_MON, &p_cap);
-        if (ret != PQOS_RETVAL_OK)
-                return;
-
-        /* update capabilities structure */
-        for (i = 0; i < p_cap->u.mon->num_events; i++) {
-                struct pqos_monitor *mon = &p_cap->u.mon->events[i];
-
-                if (supported_events & mon->type)
-                        mon->os_support = PQOS_OS_MON_RESCTRL;
-        }
-}
-
 int
 resctrl_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
 {
@@ -120,21 +88,11 @@ resctrl_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
         struct stat st;
 
         ASSERT(cpu != NULL);
-        ASSERT(cap != NULL);
+
+        UNUSED_PARAM(cap);
 
         /**
-         * Check if resctrl is mounted
-         */
-        if (stat(RESCTRL_PATH_INFO, &st) != 0) {
-                ret = resctrl_mount(PQOS_REQUIRE_CDP_OFF, PQOS_REQUIRE_CDP_OFF);
-                if (ret != PQOS_RETVAL_OK) {
-                        LOG_INFO("Unable to mount resctrl\n");
-                        return PQOS_RETVAL_RESOURCE;
-                }
-        }
-
-        /**
-         * Resctrl monitiring not supported
+         * Resctrl monitoring not supported
          */
         if (stat(RESCTRL_PATH_INFO_L3_MON, &st) != 0)
                 return PQOS_RETVAL_OK;
@@ -142,7 +100,7 @@ resctrl_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
         /**
          * Discover supported events
          */
-        fd = fopen(RESCTRL_PATH_INFO_L3_MON"/mon_features", "r");
+        fd = fopen_check_symlink(RESCTRL_PATH_INFO_L3_MON"/mon_features", "r");
         if (fd == NULL) {
                 LOG_ERROR("Failed to obtain resctrl monitoring features\n");
                 return PQOS_RETVAL_ERROR;
@@ -176,12 +134,6 @@ resctrl_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
 
         fclose(fd);
 
-        /**
-         * Update mon capabilities
-         */
-        set_mon_caps(cap);
-
-        m_cap = cap;
         m_cpu = cpu;
 
         return ret;
@@ -190,7 +142,6 @@ resctrl_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
 int
 resctrl_mon_fini(void)
 {
-        m_cap = NULL;
         m_cpu = NULL;
 
         return PQOS_RETVAL_OK;
@@ -210,10 +161,13 @@ alloc_assoc_get(const unsigned lcore, unsigned *class_id)
 {
         int ret;
         unsigned max_cos;
+        const struct pqos_cap *cap;
 
         ASSERT(class_id != NULL);
 
-        ret = resctrl_alloc_get_grps_num(m_cap, &max_cos);
+        _pqos_cap_get(&cap, NULL);
+
+        ret = resctrl_alloc_get_grps_num(cap, &max_cos);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -243,10 +197,13 @@ alloc_assoc_get_pid(const pid_t tid, unsigned *class_id)
 {
         int ret;
         unsigned max_cos;
+        const struct pqos_cap *cap;
 
         ASSERT(class_id != NULL);
 
-        ret = resctrl_alloc_get_grps_num(m_cap, &max_cos);
+        _pqos_cap_get(&cap, NULL);
+
+        ret = resctrl_alloc_get_grps_num(cap, &max_cos);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -325,7 +282,7 @@ resctrl_mon_cpumask_write(const unsigned class_id,
         resctrl_mon_group_path(class_id, resctrl_group, "/cpus", path,
                                sizeof(path));
 
-        fd = fopen(path, "w");
+        fd = fopen_check_symlink(path, "w");
         if (fd == NULL)
                 return PQOS_RETVAL_ERROR;
 
@@ -361,7 +318,7 @@ resctrl_mon_cpumask_read(const unsigned class_id,
         resctrl_mon_group_path(class_id, resctrl_group, "/cpus", path,
                                sizeof(path));
 
-        fd = fopen(path, "r");
+        fd = fopen_check_symlink(path, "r");
         if (fd == NULL)
                 return PQOS_RETVAL_ERROR;
 
@@ -426,13 +383,13 @@ resctrl_mon_read_counters(const unsigned class_id,
         }
 
         for (socket = 0; socket < sockets_num; socket++) {
-                char path[128];
+                char path[PATH_MAX];
                 FILE *fd;
                 unsigned long long counter;
 
                 snprintf(path, sizeof(path), "%s/mon_data/mon_L3_%02u/%s", buf,
                          sockets[socket], name);
-                fd = fopen(path, "r");
+                fd = fopen_check_symlink(path, "r");
                 if (fd == NULL) {
                         ret = PQOS_RETVAL_ERROR;
                         goto resctrl_mon_read_exit;
@@ -496,7 +453,7 @@ resctrl_mon_empty(const unsigned class_id,
          */
         resctrl_mon_group_path(class_id, resctrl_group, "/tasks", path,
                                sizeof(path));
-        fd = fopen(path, "r");
+        fd = fopen_check_symlink(path, "r");
         if (fd == NULL)
                 return PQOS_RETVAL_ERROR;
         /* Search tasks file for any task ID */
@@ -519,7 +476,8 @@ resctrl_mon_empty(const unsigned class_id,
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
-        fd = fopen(RESCTRL_PATH"/info/L3_MON/max_threshold_occupancy", "r");
+        fd = fopen_check_symlink(RESCTRL_PATH
+                                 "/info/L3_MON/max_threshold_occupancy", "r");
         if (fd == NULL)
                 return PQOS_RETVAL_ERROR;
         if (fscanf(fd, "%u", &max_threshold_occupancy) != 1)
@@ -528,7 +486,7 @@ resctrl_mon_empty(const unsigned class_id,
                 *empty = 0;
         fclose(fd);
 
-        return PQOS_RETVAL_OK;
+        return ret;
 }
 
 /**
@@ -731,7 +689,7 @@ resctrl_mon_assoc_get_pid(const pid_t task,
                 resctrl_mon_group_path(class_id, namelist[i]->d_name, "/tasks",
                                        path, sizeof(path));
 
-                fd = fopen(path, "r");
+                fd = fopen_check_symlink(path, "r");
                 if (fd == NULL)
                         goto resctrl_mon_assoc_get_pid_exit;
 
@@ -788,7 +746,7 @@ resctrl_mon_assoc_set_pid(const pid_t task, const char *name)
         }
 
         strncat(path, "/tasks", sizeof(path) - strlen(path) - 1);
-        fd = fopen(path, "w");
+        fd = fopen_check_symlink(path, "w");
         if (fd == NULL)
                 return PQOS_RETVAL_ERROR;
 
@@ -877,10 +835,13 @@ resctrl_mon_stop(struct pqos_mon_data *group)
         unsigned max_cos;
         unsigned cos;
         unsigned i;
+        const struct pqos_cap *cap;
 
         ASSERT(group != NULL);
 
-        ret = resctrl_alloc_get_grps_num(m_cap, &max_cos);
+        _pqos_cap_get(&cap, NULL);
+
+        ret = resctrl_alloc_get_grps_num(cap, &max_cos);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -913,7 +874,7 @@ resctrl_mon_stop(struct pqos_mon_data *group)
                         if (resctrl_alloc_task_validate(tid) !=
                             PQOS_RETVAL_OK) {
                                 LOG_DEBUG("resctrl_mon_stop: Skipping "
-                                          "non-existant PID: %d\n", tid);
+                                          "non-existent PID: %d\n", tid);
                                 continue;
                         }
                         ret = resctrl_mon_assoc_set_pid(tid, NULL);
@@ -961,11 +922,13 @@ resctrl_mon_purge(struct pqos_mon_data *group)
         unsigned max_cos;
         unsigned cos;
         int ret;
+        const struct pqos_cap *cap;
 
         ASSERT(group != NULL);
-        ASSERT(m_cap != NULL);
 
-        ret = resctrl_alloc_get_grps_num(m_cap, &max_cos);
+        _pqos_cap_get(&cap, NULL);
+
+        ret = resctrl_alloc_get_grps_num(cap, &max_cos);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -1041,11 +1004,13 @@ resctrl_mon_poll(struct pqos_mon_data *group, const enum pqos_mon_event event)
         unsigned cos;
         unsigned i;
         uint64_t old_value;
+        const struct pqos_cap *cap;
 
         ASSERT(group != NULL);
-        ASSERT(m_cap != NULL);
 
-        ret = resctrl_alloc_get_grps_num(m_cap, &max_cos);
+        _pqos_cap_get(&cap, NULL);
+
+        ret = resctrl_alloc_get_grps_num(cap, &max_cos);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -1125,11 +1090,14 @@ resctrl_mon_reset(void)
         unsigned grps;
         int num_groups;
         unsigned cos = 0;
+        const struct pqos_cap *cap;
 
         if (supported_events == 0)
                 return PQOS_RETVAL_RESOURCE;
 
-        ret = resctrl_alloc_get_grps_num(m_cap, &grps);
+        _pqos_cap_get(&cap, NULL);
+
+        ret = resctrl_alloc_get_grps_num(cap, &grps);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -1177,13 +1145,16 @@ resctrl_mon_active(unsigned *monitoring_status)
 	unsigned group_idx = 0;
 	unsigned resctrl_group_count = 0;
 	int ret;
+        const struct pqos_cap *cap;
 
 	if (supported_events == 0) {
 		*monitoring_status = 0;
 		return PQOS_RETVAL_OK;
 	}
 
-	ret = resctrl_alloc_get_grps_num(m_cap, &resctrl_group_count);
+        _pqos_cap_get(&cap, NULL);
+
+	ret = resctrl_alloc_get_grps_num(cap, &resctrl_group_count);
 	if (ret != PQOS_RETVAL_OK) {
 		LOG_ERROR("Failed to count resctrl groups");
 		return ret;
