@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2018 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2019 Intel Corporation. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -52,6 +52,9 @@
 #include "monitor.h"
 #include "alloc.h"
 #include "cap.h"
+#include "common.h"
+
+#define FILE_READ_WRITE (0600)
 
 /**
  * Default L3 CDP configuration option - don't enforce on or off
@@ -63,6 +66,11 @@ static enum pqos_cdp_config selfn_l3cdp_config = PQOS_REQUIRE_CDP_ANY;
  */
 static enum pqos_cdp_config selfn_l2cdp_config = PQOS_REQUIRE_CDP_ANY;
 
+/**
+ * Default MBA configuration option - don't enforce on or off
+ */
+static enum pqos_mba_config selfn_mba_config = PQOS_MBA_ANY;
+
 /**
  * Monitoring reset
  */
@@ -113,6 +121,11 @@ static int sel_display_verbose = 0;
  */
 enum pqos_interface sel_interface = PQOS_INTER_MSR;
 
+/**
+ * Enable displaying library version
+ */
+static int sel_print_version = 0;
+
 /**
  * @brief Function to check if a value is already contained in a table
  *
@@ -308,7 +321,7 @@ selfn_super_verbose_mode(const char *arg)
  */
 static void selfn_reset_alloc(const char *arg)
 {
-        if (arg != NULL && (strlen(arg) > 0)) {
+        if (arg != NULL && *arg != '\0') {
                 unsigned i;
                 char *tok = NULL;
                 char *saveptr = NULL;
@@ -329,6 +342,15 @@ static void selfn_reset_alloc(const char *arg)
                         {"l2cdp-any", PQOS_REQUIRE_CDP_ANY},
                 };
 
+                const struct {
+                        const char *name;
+                        enum pqos_mba_config mba;
+                } patternsmb[] = {
+                        {"mbaCtrl-on", PQOS_MBA_CTRL},
+                        {"mbaCtrl-off", PQOS_MBA_DEFAULT},
+                        {"mbaCtrl-any", PQOS_MBA_ANY},
+                };
+
                 tok = s;
                 while ((tok = strtok_r(tok, ",", &saveptr)) != NULL) {
                         unsigned valid = 0;
@@ -347,6 +369,13 @@ static void selfn_reset_alloc(const char *arg)
                                         break;
                                 }
 
+                        for (i = 0; i < DIM(patternsmb); i++)
+                                if (strcasecmp(tok, patternsmb[i].name) == 0) {
+                                        selfn_mba_config = patternsmb[i].mba;
+                                        valid = 1;
+                                        break;
+                                }
+
                         if (!valid) {
                                 printf("Unrecognized '%s' allocation "
                                        "reset option!\n", tok);
@@ -418,6 +447,18 @@ selfn_iface_os(const char *arg)
         sel_interface = PQOS_INTER_OS;
 }
 
+/**
+ * @brief Selects printing library version
+ *
+ * @param arg not used
+ */
+static void
+selfn_print_version(const char *arg)
+{
+        UNUSED_ARG(arg);
+        sel_print_version = 1;
+}
+
 /**
  * @brief Opens configuration file and parses its contents
  *
@@ -431,7 +472,7 @@ parse_config_file(const char *fname)
 
         static const struct {
                 const char *option;
-                void (*fn)(const char *);
+                void (*fn)(const char *arg);
         } optab[] = {
                 {"show-alloc:",         selfn_show_allocation },   /**< -s */
                 {"display:",            selfn_display },           /**< -d */
@@ -455,7 +496,7 @@ parse_config_file(const char *fname)
         FILE *fp = NULL;
         char cb[256];
 
-        fp = fopen(fname, "r");
+        fp = fopen_check_symlink(fname, "r");
         if (fp == NULL)
                 parse_error(fname, "cannot open configuration file!");
 
@@ -517,6 +558,7 @@ parse_config_file(const char *fname)
 static const char *m_cmd_name = "pqos";                     /**< command name */
 static const char help_printf_short[] =
         "Usage: %s [-h] [--help] [-v] [--verbose] [-V] [--super-verbose]\n"
+        "       %s [-w] [--version]\n"
         "          [-l FILE] [--log-file=FILE] [-I] [--iface-os]\n"
         "       %s [-s] [--show]\n"
         "       %s [-d] [--display] [-D] [--display-verbose]\n"
@@ -528,6 +570,7 @@ static const char help_printf_short[] =
         "          [-o FILE] [--mon-file=FILE]\n"
         "          [-u TYPE] [--mon-file-type=TYPE]\n"
         "          [-r] [--mon-reset]\n"
+        "          [-P] [--percent-llc]\n"
         "       %s [-e CLASSDEF] [--alloc-class=CLASSDEF]\n"
         "          [-a CLASS2ID] [--alloc-assoc=CLASS2ID]\n"
         "       %s [-R] [--alloc-reset]\n"
@@ -540,6 +583,7 @@ static const char help_printf_long[] =
         "  -h, --help                  help page\n"
         "  -v, --verbose               verbose mode\n"
         "  -V, --super-verbose         super-verbose mode\n"
+        "  -w, --version               show PQoS library version\n"
         "  -s, --show                  show current PQoS configuration\n"
         "  -d, --display               display supported capabilities\n"
         "  -D, --display-verbose       display supported capabilities in verbose mode\n"
@@ -550,10 +594,11 @@ static const char help_printf_long[] =
         "          CLASSDEF format is 'TYPE:ID=DEFINITION;'.\n"
         "          To specify specific resources 'TYPE[@RESOURCE_ID]:ID=DEFINITION;'.\n"
         "          Examples: 'llc:0=0xffff;llc:1=0x00ff;llc@0-1:2=0xff00',\n"
-	"                    'llc:0d=0xfff;llc:0c=0xfff00',\n"
+        "                    'llc:0d=0xfff;llc:0c=0xfff00',\n"
         "                    'l2:2=0x3f;l2@2:1=0xf',\n"
-        "                    'l2:2d=0xf;l2:2c=0xc,\n"
-        "                    'mba:1=30;mba@1:3=80'.\n"
+        "                    'l2:2d=0xf;l2:2c=0xc',\n"
+        "                    'mba:1=30;mba@1:3=80',\n"
+        "                    'mba_max:1=4000;mba_max@1:3=6000'.\n"
         "  -a CLASS2ID, --alloc-assoc=CLASS2ID\n"
         "          associate cores/tasks with an allocation class.\n"
         "          CLASS2ID format is 'TYPE:ID=CORE_LIST/TASK_LIST'.\n"
@@ -563,8 +608,9 @@ static const char help_printf_long[] =
         "  -R [CONFIG[,CONFIG]], --alloc-reset[=CONFIG[,CONFIG]]\n"
         "          reset allocation configuration (L2/L3 CAT & MBA)\n"
         "          CONFIG can be: l3cdp-on, l3cdp-off, l3cdp-any,\n"
-        "                         l2cdp-on, l2cdp-off, l2cdp-any\n"
-        "          (default l3cdp-any,l2cdp-any).\n"
+        "                         l2cdp-on, l2cdp-off, l2cdp-any,\n"
+        "                         mbaCtrl-on, mbaCtrl-off, mbaCtrl-any\n"
+        "          (default l3cdp-any,l2cdp-any,mbaCtrl-any).\n"
         "  -m EVTCORES, --mon-core=EVTCORES\n"
         "          select cores and events for monitoring.\n"
         "          EVTCORES format is 'EVENT:CORE_LIST'.\n"
@@ -582,6 +628,9 @@ static const char help_printf_long[] =
         "               Requires Linux and kernel versions 4.10 and newer.\n"
         "               The -I option must be used for PID monitoring.\n"
         "               Processes and cores cannot be monitored together.\n"
+        "  -P, --percent-llc\n"
+        "         Displays LLC as percentage value (by default LLC is displayed\n"
+        "         in kilobytes if this parameter is not used)\n"
         "  -o FILE, --mon-file=FILE    output monitored data in a FILE\n"
         "  -u TYPE, --mon-file-type=TYPE\n"
         "          select output file format type for monitored data.\n"
@@ -612,11 +661,25 @@ static void print_help(const int is_long)
 {
         printf(help_printf_short,
                m_cmd_name, m_cmd_name, m_cmd_name, m_cmd_name, m_cmd_name,
-               m_cmd_name, m_cmd_name, m_cmd_name);
+               m_cmd_name, m_cmd_name, m_cmd_name, m_cmd_name);
         if (is_long)
                 printf("%s", help_printf_long);
 }
 
+/**
+ * @brief Displays PQoS library version
+ *
+ * @param [in] p_cap platform capabilities
+ */
+static void print_lib_version(const struct pqos_cap *p_cap)
+{
+        int major = p_cap->version / 10000;
+        int minor = (p_cap->version % 10000) / 100;
+        int patch = p_cap->version % 100;
+
+        printf("PQoS Library version: %d.%d.%d\n", major, minor, patch);
+}
+
 static struct option long_cmd_opts[] = {
         {"help",            no_argument,       0, 'h'},
         {"log-file",        required_argument, 0, 'l'},
@@ -640,6 +703,8 @@ static struct option long_cmd_opts[] = {
         {"verbose",         no_argument,       0, 'v'},
         {"super-verbose",   no_argument,       0, 'V'},
         {"iface-os",        no_argument,       0, 'I'},
+        {"percent-llc",     no_argument,       0, 'P'},
+        {"version",         no_argument,       0, 'w'},
         {0, 0, 0, 0} /* end */
 };
 
@@ -660,15 +725,18 @@ int main(int argc, char **argv)
         memset(&cfg, 0, sizeof(cfg));
 
         while ((cmd = getopt_long(argc, argv,
-                                  ":Hhf:i:m:Tt:l:o:u:e:c:a:p:sdDrvVIR:",
+                                  ":Hhwf:i:m:Tt:l:o:u:e:c:a:p:sdDrvVIPR:",
                                   long_cmd_opts, &opt_index)) != -1) {
                 switch (cmd) {
                 case 'h':
                         print_help(1);
                         return EXIT_SUCCESS;
                 case 'H':
-                        profile_l3ca_list(stdout);
+                        profile_l3ca_list();
                         return EXIT_SUCCESS;
+                case 'w':
+                        selfn_print_version(NULL);
+                        break;
                 case 'f':
                         if (sel_config_file != NULL) {
                                 printf("Only one config file argument is "
@@ -682,6 +750,7 @@ int main(int argc, char **argv)
                         selfn_monitor_interval(optarg);
                         break;
                 case 'p':
+                        pid_flag = 1;
                         if (optarg != NULL && *optarg == '-') {
                                 /**
                                  * Next switch option wrongly assumed to be
@@ -696,7 +765,9 @@ int main(int argc, char **argv)
                                 break;
                         }
                         selfn_monitor_pids(optarg);
-                        pid_flag = 1;
+                        break;
+                case 'P':
+                        selfn_monitor_set_llc_percent();
                         break;
                 case 'm':
                         selfn_monitor_cores(optarg);
@@ -726,11 +797,11 @@ int main(int argc, char **argv)
                         if (optarg != NULL) {
                                 if (*optarg == '-') {
                                         /**
-                                        * Next switch option wrongly assumed
-                                        * to be argument to '-R'.
-                                        * Pass NULL as argument to '-R' function
-                                        * and rewind \a optind.
-                                        */
+                                         * Next switch option wrongly assumed
+                                         * to be argument to '-R'.
+                                         * Pass NULL as argument to
+                                         * '-R' function and rewind \a optind.
+                                         */
                                         selfn_reset_alloc(NULL);
                                         optind--;
                                 } else
@@ -813,7 +884,7 @@ int main(int argc, char **argv)
                 cfg.fd_log = STDOUT_FILENO;
         } else {
                 cfg.fd_log = open(sel_log_file, O_WRONLY|O_CREAT,
-                                  S_IRUSR|S_IWUSR);
+                                  FILE_READ_WRITE);
                 if (cfg.fd_log == -1) {
                         printf("Error opening %s log file!\n", sel_log_file);
                         exit_val = EXIT_FAILURE;
@@ -870,6 +941,12 @@ int main(int argc, char **argv)
                 goto error_exit_2;
         }
 
+        if (sel_print_version) {
+                print_lib_version(p_cap);
+                exit_val = EXIT_SUCCESS;
+                goto error_exit_2;
+        }
+
         if (sel_mon_reset && cap_mon != NULL) {
                 ret = pqos_mon_reset();
                 if (sel_interface != PQOS_INTER_MSR &&
@@ -891,7 +968,8 @@ int main(int argc, char **argv)
                  * Reset allocation configuration to after-reset state and exit
                  */
                 ret = pqos_alloc_reset(selfn_l3cdp_config,
-                                       selfn_l2cdp_config);
+                                       selfn_l2cdp_config,
+                                       selfn_mba_config);
                 if (ret != PQOS_RETVAL_OK) {
                         exit_val = EXIT_FAILURE;
                         printf("Allocation reset failed!\n");
