@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2017 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2019 Intel Corporation. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,6 +50,8 @@
 #include <dirent.h>                                     /**< for dir list*/
 
 #include "pqos.h"
+
+#include "common.h"
 #include "main.h"
 #include "monitor.h"
 
@@ -61,6 +63,7 @@
 #define PID_COL_CORE  (39) /**< col for core number in /proc/pid/stat*/
 #define PID_CPU_TIME_DELAY_USEC (1200000) /**< delay for cpu stats */
 #define TOP_PROC_MAX (10) /**< maximum number of top-pids to be handled*/
+#define NUM_TIDS_MAX (128) /**< maximum number of TIDs */
 
 /**
  * Local data structures
@@ -178,6 +181,23 @@ struct slist {
         struct slist *next; /**< ptr to next list element */
 };
 
+/**
+ * Stores display format for LLC (kilobytes/percent)
+ */
+static enum llc_format {
+        LLC_FORMAT_KILOBYTES = 0,
+        LLC_FORMAT_PERCENT
+} sel_llc_format = LLC_FORMAT_KILOBYTES;
+
+/**
+ * Manages llc entry with data to be displayed with current llc_format
+ * (value may be displayed as kilobytes value or as percent of total cache)
+ */
+struct llc_entry_data {
+        double val;
+        enum llc_format format;
+};
+
 /**
  * @brief Scale byte value up to KB
  *
@@ -219,14 +239,24 @@ static inline int process_mode(void)
  * @param buf buffer string will be copied into
  * @param buf_len length of buffer
  * @param val value to be translated
+ *
+ * @return length of generated string
+ * @retval < 0 on error
  */
-static void
+static int
 uinttostr_noalloc(char *buf, const int buf_len, const unsigned val)
 {
         ASSERT(buf != NULL);
+        int ret;
 
         memset(buf, 0, buf_len);
-        snprintf(buf, buf_len, "%u", val);
+        ret = snprintf(buf, buf_len, "%u", val);
+
+        /* Return -1 when output was truncated */
+        if (ret >= buf_len)
+                ret = -1;
+
+        return ret;
 }
 
 /**
@@ -241,7 +271,7 @@ static char *uinttostr(const unsigned val)
 {
         char buf[16], *str = NULL;
 
-        uinttostr_noalloc(buf, sizeof(buf), val);
+        (void)uinttostr_noalloc(buf, sizeof(buf), val);
         selfn_strdup(&str, buf);
 
         return str;
@@ -359,13 +389,11 @@ strtogrps(char *s,
                 return group_count;
 
         while ((non_grp = strsep(&s, "[")) != NULL) {
-
-                int ret;
                 /**
                  * Ungrouped cores/pids
                  */
-                if ((strlen(non_grp)) > 0) {
-                        /* for seperate cores/pids - each will get his own
+                if (*non_grp != '\0') {
+                        /* for separate cores/pids - each will get his own
                          * group so strlisttotab result is treated as the
                          * number of new groups
                          */
@@ -378,6 +406,7 @@ strtogrps(char *s,
                         /* set group info */
                         for (i = 0; i < new_groups_count; i++) {
                                 char *desc = uinttostr((unsigned)cbuf[i]);
+                                int ret;
 
                                 if (ctab != NULL)
                                         ret = set_cgrp(&ctab[group_count],
@@ -400,6 +429,7 @@ strtogrps(char *s,
                 if (grp != NULL) {
                         char *desc = NULL;
                         unsigned element_count;
+                        int ret;
 
                         if (group_count >= max)
                                 return -1;
@@ -635,6 +665,11 @@ void selfn_monitor_file(const char *arg)
         selfn_strdup(&sel_output_file, arg);
 }
 
+void selfn_monitor_set_llc_percent(void)
+{
+        sel_llc_format = LLC_FORMAT_PERCENT;
+}
+
 void selfn_monitor_cores(const char *arg)
 {
         char *cp = NULL, *str = NULL;
@@ -643,7 +678,7 @@ void selfn_monitor_cores(const char *arg)
         if (arg == NULL)
                 parse_error(arg, "NULL pointer!");
 
-        if (strlen(arg) <= 0)
+        if (*arg == '\0')
                 parse_error(arg, "Empty string!");
 
         selfn_strdup(&cp, arg);
@@ -665,8 +700,7 @@ int monitor_setup(const struct pqos_cpuinfo *cpu_info,
 {
         unsigned i;
         int ret;
-        enum pqos_mon_event all_core_evts = (enum pqos_mon_event)0;
-        enum pqos_mon_event all_pid_evts = (enum pqos_mon_event)0;
+        enum pqos_mon_event all_evts = (enum pqos_mon_event)0;
         const enum pqos_mon_event evt_all =
                 (enum pqos_mon_event)PQOS_MON_EVENT_ALL;
 
@@ -700,9 +734,9 @@ int monitor_setup(const struct pqos_cpuinfo *cpu_info,
         } else {
                 if (strcasecmp(sel_output_type, "xml") == 0 ||
                     strcasecmp(sel_output_type, "csv") == 0)
-                        fp_monitor = fopen(sel_output_file, "w+");
+                        fp_monitor = fopen_check_symlink(sel_output_file, "w+");
                 else
-                        fp_monitor = fopen(sel_output_file, "a");
+                        fp_monitor = fopen_check_symlink(sel_output_file, "a");
                 if (fp_monitor == NULL) {
                         perror("Monitoring output file open error:");
                         printf("Error opening '%s' output file!\n",
@@ -721,17 +755,14 @@ int monitor_setup(const struct pqos_cpuinfo *cpu_info,
         for (i = 0; i < cap_mon->u.mon->num_events; i++) {
                 struct pqos_monitor *mon = &cap_mon->u.mon->events[i];
 
-                if (sel_interface == PQOS_INTER_MSR || mon->os_support)
-                        all_core_evts |= mon->type;
-                if (mon->os_support)
-                        all_pid_evts |= mon->type;
+                all_evts |= mon->type;
         }
         /**
          * If no cores and events selected through command line
          * by default let's monitor all cores
          */
         if (sel_monitor_num == 0 && sel_process_num == 0) {
-                sel_events_max = all_core_evts;
+                sel_events_max = all_evts;
                 for (i = 0; i < cpu_info->num_cores; i++) {
                         unsigned lcore  = cpu_info->cores[i].lcore;
                         uint64_t core = (uint64_t)lcore;
@@ -769,13 +800,13 @@ int monitor_setup(const struct pqos_cpuinfo *cpu_info,
 
                         /* check if all available events were selected */
                         if (cg->events == evt_all) {
-                                cg->events = all_core_evts;
-                                sel_events_max |= all_core_evts;
+                                cg->events = all_evts;
+                                sel_events_max |= all_evts;
                         } else {
-                                if (all_core_evts & PQOS_PERF_EVENT_IPC)
+                                if (all_evts & PQOS_PERF_EVENT_IPC)
                                         cg->events |= (enum pqos_mon_event)
                                                 PQOS_PERF_EVENT_IPC;
-                                if (all_core_evts & PQOS_PERF_EVENT_LLC_MISS)
+                                if (all_evts & PQOS_PERF_EVENT_LLC_MISS)
                                         cg->events |= (enum pqos_mon_event)
                                                 PQOS_PERF_EVENT_LLC_MISS;
                         }
@@ -802,9 +833,7 @@ int monitor_setup(const struct pqos_cpuinfo *cpu_info,
                                  * Stop mon groups that are already started
                                  */
                                 for (j = 0; j < i; j++) {
-                                        struct core_group *cg =
-                                                &sel_monitor_core_tab[j];
-
+                                        cg = &sel_monitor_core_tab[j];
                                         pqos_mon_stop(cg->pgrp);
                                 }
                                 return -1;
@@ -819,13 +848,13 @@ int monitor_setup(const struct pqos_cpuinfo *cpu_info,
 
                         /* check if all available events were selected */
                         if (pg->events == evt_all) {
-                                pg->events = all_pid_evts;
-                                sel_events_max |= all_pid_evts;
+                                pg->events = all_evts;
+                                sel_events_max |= all_evts;
                         } else {
-                                if (all_pid_evts & PQOS_PERF_EVENT_IPC)
+                                if (all_evts & PQOS_PERF_EVENT_IPC)
                                         pg->events |= (enum pqos_mon_event)
                                                 PQOS_PERF_EVENT_IPC;
-                                if (all_pid_evts & PQOS_PERF_EVENT_LLC_MISS)
+                                if (all_evts & PQOS_PERF_EVENT_LLC_MISS)
                                         pg->events |= (enum pqos_mon_event)
                                                 PQOS_PERF_EVENT_LLC_MISS;
                         }
@@ -847,9 +876,7 @@ int monitor_setup(const struct pqos_cpuinfo *cpu_info,
                                  * Stop mon groups that are already started
                                  */
                                 for (j = 0; j < i; j++) {
-                                        struct pid_group *pg =
-                                                &sel_monitor_pid_tab[j];
-
+                                        pg = &sel_monitor_pid_tab[j];
                                         pqos_mon_stop(pg->pgrp);
                                 }
                                 return -1;
@@ -1019,7 +1046,7 @@ selfn_monitor_pids(const char *arg)
         if (arg == NULL)
                 parse_error(arg, "NULL pointer!");
 
-        if (strlen(arg) <= 0)
+        if (*arg == '\0')
                 parse_error(arg, "Empty string!");
 
         selfn_strdup(&cp, arg);
@@ -1240,7 +1267,9 @@ get_pid_core_num(const pid_t pid, unsigned *core)
                 return -1;
 
         memset(core_s, 0, sizeof(core_s));
-        uinttostr_noalloc(pid_s, sizeof(pid_s), pid);
+        ret = uinttostr_noalloc(pid_s, sizeof(pid_s), pid);
+        if (ret < 0)
+                return -1;
 
         ret = get_pid_stat_val(pid_s, PID_COL_CORE, sizeof(core_s), core_s);
         if (ret != 0)
@@ -1280,7 +1309,7 @@ unsigned_cmp(const void *a, const void *b)
 }
 
 /**
- * @brief Function to return a comma seperated list of all cores that PIDs
+ * @brief Function to return a comma separated list of all cores that PIDs
  *        in \a mon_data last ran on.
  *
  * @param mon_data struct with info on the group of pids to be monitored
@@ -1299,34 +1328,56 @@ get_pid_cores(const struct pqos_mon_data *mon_data, char *cores_s,
         int cores_s_len = 0;
         int comma_len = 1;
         unsigned *cores;
+        int ret;
+        pid_t *tids;
+        unsigned num_tids;
+        int result = 0;
 
         ASSERT(mon_data != NULL);
         ASSERT(cores_s != NULL);
 
-        cores = calloc(mon_data->num_pids, sizeof(*cores));
+        tids = calloc(NUM_TIDS_MAX, sizeof(*tids));
 
-        if (cores == NULL) {
+        if (tids == NULL) {
                 printf("Error allocating memory\n");
                 return -1;
         }
 
-        for (i = 0; i < mon_data->num_pids; i++)
-                if (get_pid_core_num(mon_data->pids[i], &cores[i]) == -1) {
-                        free(cores);
-                        return -1;
+        ret = pqos_mon_tids_get(mon_data, tids, NUM_TIDS_MAX, &num_tids);
+
+        if (ret != PQOS_RETVAL_OK) {
+                free(tids);
+                return -1;
+        }
+
+        cores = calloc(num_tids, sizeof(*cores));
+
+        if (cores == NULL) {
+                printf("Error allocating memory\n");
+                free(tids);
+                return -1;
+        }
+
+        for (i = 0; i < num_tids; i++)
+                if (get_pid_core_num(tids[i], &cores[i]) == -1) {
+                        result = -1;
+                        goto free_memory;
                 }
 
-        qsort(cores, mon_data->num_pids, sizeof(*cores), unsigned_cmp);
+        qsort(cores, num_tids, sizeof(*cores), unsigned_cmp);
 
-        for (i = 0; i < mon_data->num_pids; i++) {
+        for (i = 0; i < num_tids; i++) {
 
                 /* check for duplicate cores and skips them*/
                 if (i != 0 && cores[i] == cores[i-1])
                         continue;
 
-                uinttostr_noalloc(core, sizeof(core), cores[i]);
+                str_len = uinttostr_noalloc(core, sizeof(core), cores[i]);
+                if (str_len < 0) {
+                        result = -1;
+                        goto free_memory;
+                }
 
-                str_len = strlen(core);
                 cores_s_len = strlen(cores_s);
 
                 if (i != 0 && (cores_s_len + str_len + comma_len) < len) {
@@ -1335,13 +1386,15 @@ get_pid_cores(const struct pqos_mon_data *mon_data, char *cores_s,
                 } else if (i == 0 && (cores_s_len + str_len) < len)
                         strcat(cores_s, core);
                 else {
-                        free(cores);
-                        return -1;
+                        result = -1;
+                        goto free_memory;
                 }
         }
 
+free_memory:
         free(cores);
-        return 0;
+        free(tids);
+        return result;
 }
 
 /**
@@ -1921,7 +1974,7 @@ static size_t
 fillin_text_column(const double val, char data[], const size_t sz_data,
                    const int is_monitored, const int is_column_present)
 {
-        const char blank_column[] = "            ";
+        static const char blank_column[] = "            ";
         size_t offset = 0;
 
         if (sz_data <= sizeof(blank_column))
@@ -2021,14 +2074,14 @@ fillin_csv_column(const double val, char data[], const size_t sz_data,
  *
  * @param fp pointer to file to direct output
  * @param mon_data pointer to pqos_mon_data structure
- * @param llc LLC occupancy data
+ * @param llc_entry LLC occupancy data structure
  * @param mbr remote memory bandwidth data
  * @param mbl local memory bandwidth data
  */
 static void
 print_text_row(FILE *fp,
                struct pqos_mon_data *mon_data,
-               const double llc,
+               const struct llc_entry_data *llc_entry,
                const double mbr,
                const double mbl)
 {
@@ -2039,10 +2092,12 @@ print_text_row(FILE *fp,
 
         ASSERT(fp != NULL);
         ASSERT(mon_data != NULL);
+        ASSERT(llc_entry != NULL);
 
         memset(data, 0, sz_data);
 
-        offset += fillin_text_column(llc, data + offset, sz_data - offset,
+        offset += fillin_text_column(llc_entry->val, data + offset,
+                                     sz_data - offset,
                                      mon_data->event & PQOS_MON_EVENT_L3_OCCUP,
                                      sel_events_max & PQOS_MON_EVENT_L3_OCCUP);
 
@@ -2066,7 +2121,7 @@ print_text_row(FILE *fp,
                 if (get_pid_cores(mon_data, core_list,
                                   sizeof(core_list)) == -1) {
                         memset(core_list, 0, sizeof(core_list));
-                        strcat(core_list, "err");
+                        strncat(core_list, "err", sizeof(core_list) - 1);
                 }
 
                 fprintf(fp, "\n%8.8s %8.8s %6.2f %7uk%s",
@@ -2083,14 +2138,14 @@ print_text_row(FILE *fp,
  * @param fp pointer to file to direct output
  * @param time pointer to string containing time data
  * @param mon_data pointer to pqos_mon_data structure
- * @param llc LLC occupancy data
+ * @param llc_entry LLC occupancy data structure
  * @param mbr remote memory bandwidth data
  * @param mbl local memory bandwidth data
  */
 static void
 print_xml_row(FILE *fp, char *time,
               struct pqos_mon_data *mon_data,
-              const double llc,
+              const struct llc_entry_data *llc_entry,
               const double mbr,
               const double mbl)
 {
@@ -2102,11 +2157,16 @@ print_xml_row(FILE *fp, char *time,
         ASSERT(fp != NULL);
         ASSERT(time != NULL);
         ASSERT(mon_data != NULL);
+        ASSERT(llc_entry != NULL);
 
-        offset += fillin_xml_column(llc, data + offset, sz_data - offset,
+        const char *l3_text = (llc_entry->format == LLC_FORMAT_KILOBYTES) ?
+                "l3_occupancy_kB" : "l3_occupancy_percent";
+
+        offset += fillin_xml_column(llc_entry->val, data + offset,
+                                    sz_data - offset,
                                     mon_data->event & PQOS_MON_EVENT_L3_OCCUP,
                                     sel_events_max & PQOS_MON_EVENT_L3_OCCUP,
-                                    "l3_occupancy_kB");
+                                    l3_text);
 
         offset += fillin_xml_column(mbl, data + offset, sz_data - offset,
                                     mon_data->event & PQOS_MON_EVENT_LMEM_BW,
@@ -2140,7 +2200,7 @@ print_xml_row(FILE *fp, char *time,
                 if (get_pid_cores(mon_data, core_list,
                                   sizeof(core_list)) == -1) {
                         memset(core_list, 0, sizeof(core_list));
-                        strcat(core_list, "err");
+                        strncat(core_list, "err", sizeof(core_list) - 1);
                 }
 
                 fprintf(fp,
@@ -2169,14 +2229,14 @@ print_xml_row(FILE *fp, char *time,
  * @param fp pointer to file to direct output
  * @param time pointer to string containing time data
  * @param mon_data pointer to pqos_mon_data structure
- * @param llc LLC occupancy data
+ * @param llc_entry LLC occupancy data structure
  * @param mbr remote memory bandwidth data
  * @param mbl local memory bandwidth data
  */
 static void
 print_csv_row(FILE *fp, char *time,
               struct pqos_mon_data *mon_data,
-              const double llc,
+              const struct llc_entry_data *llc_entry,
               const double mbr,
               const double mbl)
 {
@@ -2188,10 +2248,12 @@ print_csv_row(FILE *fp, char *time,
         ASSERT(fp != NULL);
         ASSERT(time != NULL);
         ASSERT(mon_data != NULL);
+        ASSERT(llc_entry != NULL);
 
         memset(data, 0, sz_data);
 
-        offset += fillin_csv_column(llc, data + offset, sz_data - offset,
+        offset += fillin_csv_column(llc_entry->val, data + offset,
+                                    sz_data - offset,
                                     mon_data->event & PQOS_MON_EVENT_L3_OCCUP,
                                     sel_events_max & PQOS_MON_EVENT_L3_OCCUP);
 
@@ -2216,7 +2278,7 @@ print_csv_row(FILE *fp, char *time,
                 if (get_pid_cores(mon_data, core_list,
                                   sizeof(core_list)) == -1) {
                         memset(core_list, 0, sizeof(core_list));
-                        strcat(core_list, "err");
+                        strncat(core_list, "err", sizeof(core_list) - 1);
                 }
 
                 fprintf(fp,
@@ -2236,12 +2298,14 @@ print_csv_row(FILE *fp, char *time,
  * @param isxml true if XML output selected
  * @param istext true is TEXT output selected
  * @param iscsv true is CSV output selected
+ * @param format llc_format representation mode (kilobytes or percent)
  */
 static void
 build_header_row(char *hdr, const size_t sz_hdr,
                  const int isxml,
                  const int istext,
-                 const int iscsv)
+                 const int iscsv,
+                 const enum llc_format format)
 {
         ASSERT(hdr != NULL && sz_hdr > 0);
         memset(hdr, 0, sz_hdr);
@@ -2255,8 +2319,14 @@ build_header_row(char *hdr, const size_t sz_hdr,
                 else
                         strncpy(hdr, "     PID     CORE    IPC   MISSES",
                                 sz_hdr - 1);
-                if (sel_events_max & PQOS_MON_EVENT_L3_OCCUP)
-                        strncat(hdr, "     LLC[KB]", sz_hdr - strlen(hdr) - 1);
+                if (sel_events_max & PQOS_MON_EVENT_L3_OCCUP) {
+                        if (format == LLC_FORMAT_KILOBYTES)
+                                strncat(hdr, "     LLC[KB]",
+                                        sz_hdr - strlen(hdr) - 1);
+                        else
+                                strncat(hdr, "      LLC[%]",
+                                        sz_hdr - strlen(hdr) - 1);
+                }
                 if (sel_events_max & PQOS_MON_EVENT_LMEM_BW)
                         strncat(hdr, "   MBL[MB/s]", sz_hdr - strlen(hdr) - 1);
                 if (sel_events_max & PQOS_MON_EVENT_RMEM_BW)
@@ -2270,8 +2340,14 @@ build_header_row(char *hdr, const size_t sz_hdr,
                 else
                         strncpy(hdr, "Time,PID,Core,IPC,LLC Misses",
                                 sz_hdr - 1);
-                if (sel_events_max & PQOS_MON_EVENT_L3_OCCUP)
-                        strncat(hdr, ",LLC[KB]", sz_hdr - strlen(hdr) - 1);
+                if (sel_events_max & PQOS_MON_EVENT_L3_OCCUP) {
+                        if (format == LLC_FORMAT_KILOBYTES)
+                                strncat(hdr, ",LLC[KB]",
+                                        sz_hdr - strlen(hdr) - 1);
+                        else
+                                strncat(hdr, ",LLC[%]",
+                                        sz_hdr - strlen(hdr) - 1);
+                }
                 if (sel_events_max & PQOS_MON_EVENT_LMEM_BW)
                         strncat(hdr, ",MBL[MB/s]", sz_hdr - strlen(hdr) - 1);
                 if (sel_events_max & PQOS_MON_EVENT_RMEM_BW)
@@ -2354,6 +2430,77 @@ static long timeval_to_usec(const struct timeval *tv)
         return ((long)tv->tv_usec) + ((long)tv->tv_sec * 1000000L);
 }
 
+/**
+ * @brief Gets total l3 cache value
+ *
+ * @param[in] p_cache_size pointer to cache-size value to be filled.
+ *            It must not be NULL.
+ *
+ * @return PQOS_RETVAL_OK on success or error code in case of failure
+ */
+static int get_cache_size(unsigned *p_cache_size)
+{
+        const struct pqos_cpuinfo *p_cpu = NULL;
+        int ret;
+
+        if (p_cache_size == NULL)
+                return PQOS_RETVAL_PARAM;
+
+        ret = pqos_cap_get(NULL, &p_cpu);
+        if (ret != PQOS_RETVAL_OK) {
+                printf("Error retrieving PQoS capabilities!\n");
+                return ret;
+        }
+
+        if (p_cpu == NULL)
+                return PQOS_RETVAL_ERROR;
+
+        *p_cache_size = p_cpu->l3.total_size;
+
+        return PQOS_RETVAL_OK;
+}
+
+/**
+ * @brief Fills llc_entry_data structure with percentage or kilobytes
+ *        cache usage representation
+ *
+ * @param[in] llc_entry pointer to llc_entry_data structure to be filled.
+ *            It must not be NULL
+ * @param llc_bytes amount of cache(in bytes) used by core
+ * @param cache_total total l3 cache size
+ * @param format enum representing percentage or kilobyte format used
+ *        for showing cache occupancy for core
+ *
+ * @return PQOS_RETVAL_OK on success or error code otherwise
+ */
+static int get_llc_entry(struct llc_entry_data *llc_entry,
+                         uint64_t llc_bytes,
+                         unsigned cache_total,
+                         enum llc_format format)
+{
+        if (llc_entry == NULL)
+                return PQOS_RETVAL_PARAM;
+
+        if (cache_total == 0)
+                return PQOS_RETVAL_PARAM;
+
+        switch (format) {
+        case LLC_FORMAT_KILOBYTES:
+                llc_entry->val = bytes_to_kb(llc_bytes);
+                break;
+        case LLC_FORMAT_PERCENT:
+                llc_entry->val = llc_bytes*100/cache_total;
+                break;
+        default:
+                printf("Incorrect llc_format: %i\n", format);
+                return PQOS_RETVAL_PARAM;
+        }
+
+        llc_entry->format = format;
+
+        return PQOS_RETVAL_OK;
+}
+
 void monitor_loop(void)
 {
 #define TERM_MIN_NUM_LINES 3
@@ -2361,12 +2508,12 @@ void monitor_loop(void)
         const long interval =
                 (long)sel_mon_interval * 100000LL; /* interval in [us] units */
         struct timeval tv_start, tv_s;
-        int ret = PQOS_RETVAL_OK;
         const int istty = isatty(fileno(fp_monitor));
         const int istext = !strcasecmp(sel_output_type, "text");
         const int isxml = !strcasecmp(sel_output_type, "xml");
         const int iscsv = !strcasecmp(sel_output_type, "csv");
         const size_t sz_header = 128;
+        unsigned cache_size;
         char header[sz_header];
         unsigned mon_number = 0, display_num = 0;
         struct pqos_mon_data **mon_data = NULL, **mon_grps = NULL;
@@ -2377,6 +2524,11 @@ void monitor_loop(void)
                 return;
         }
 
+        if (get_cache_size(&cache_size) != PQOS_RETVAL_OK) {
+                printf("Error during getting L3 cache size\n");
+                return;
+        }
+
         mon_number = get_mon_arrays(&mon_grps, &mon_data);
         display_num = mon_number;
 
@@ -2410,7 +2562,8 @@ void monitor_loop(void)
         /**
          * Build the header
          */
-        build_header_row(header, sz_header, isxml, istext, iscsv);
+        build_header_row(header, sz_header, isxml, istext, iscsv,
+                         sel_llc_format);
 
         if (iscsv)
                 fprintf(fp_monitor, "%s\n", header);
@@ -2424,6 +2577,7 @@ void monitor_loop(void)
                 unsigned i = 0;
                 long usec_start = 0, usec_end = 0, usec_diff = 0;
                 char cb_time[64];
+                int ret;
 
                 ret = pqos_mon_poll(mon_grps, mon_number);
                 if (ret != PQOS_RETVAL_OK) {
@@ -2462,19 +2616,29 @@ void monitor_loop(void)
                 for (i = 0; i < display_num; i++) {
                         const struct pqos_event_values *pv =
                                  &mon_data[i]->values;
-                        double llc = bytes_to_kb(pv->llc);
+
+                        struct llc_entry_data llc_entry;
+                        int ret = get_llc_entry(&llc_entry, pv->llc,
+                                                cache_size,
+                                                sel_llc_format);
+                        if (ret != PQOS_RETVAL_OK) {
+                                printf("Could not fill llc_entry data!\n");
+                                stop_monitoring_loop = 1;
+                                break;
+                        }
+
                         double mbr = bytes_to_mb(pv->mbm_remote_delta) * coeff;
                         double mbl = bytes_to_mb(pv->mbm_local_delta) * coeff;
 
                         if (istext)
                                 print_text_row(fp_monitor, mon_data[i],
-                                               llc, mbr, mbl);
+                                               &llc_entry, mbr, mbl);
                         if (isxml)
                                 print_xml_row(fp_monitor, cb_time, mon_data[i],
-                                              llc, mbr, mbl);
+                                              &llc_entry, mbr, mbl);
                         if (iscsv)
                                 print_csv_row(fp_monitor, cb_time, mon_data[i],
-                                              llc, mbr, mbl);
+                                              &llc_entry, mbr, mbl);
                 }
                 if (!istty && istext)
                         fputs("\n", fp_monitor);
@@ -2552,4 +2716,3 @@ void monitor_cleanup(void)
                 free(sel_output_type);
         sel_output_type = NULL;
 }
-
