@@ -1,7 +1,7 @@
 /*
  *   BSD LICENSE
  *
- *   Copyright(c) 2016-2018 Intel Corporation. All rights reserved.
+ *   Copyright(c) 2016-2019 Intel Corporation. All rights reserved.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -44,9 +44,8 @@
 extern "C" {
 #endif
 
-#define RDT_MAX_SOCKETS 8
-#define RDT_MAX_L2IDS   32
 #define RDT_MAX_PIDS    128
+#define MAX_OPTARG_LEN  64
 
 #ifndef MIN
 /**
@@ -145,7 +144,6 @@ struct rdt_config {
 	struct pqos_l2ca l2;	/**< L2 configuration */
 	struct pqos_mba mba;	/**< MBA configuretion */
         int pid_cfg;            /**< associate PIDs to this cfg */
-        uint64_t mba_max;       /**< max bandwidth in MBps */
 };
 
 /* rdtset command line configuration structure */
@@ -158,7 +156,8 @@ struct rdtset {
 	cpu_set_t reset_cpuset;		/**< List of CPUs to reset COS assoc */
 	unsigned sudo_keep:1,		/**< don't drop elevated privileges */
 		 verbose:1,		/**< be verbose */
-		 command:1;		/**< command to be executed detected */
+		 command:1,		/**< command to be executed detected */
+		 show_version:1;	/**< print library version */
 	enum pqos_interface interface;  /**< pqos interface to use */
 };
 
