@@ -56,15 +56,14 @@ parse_cpu(const char *cpustr)
 int
 set_affinity(pid_t pid)
 {
-	int ret;
+	int ret = -1;
 	cpu_set_t cpumask;
 
 	/* Set affinity */
 #ifdef __linux__
 	ret = sched_setaffinity(pid, sizeof(g_cfg.cpu_aff_cpuset),
 		&g_cfg.cpu_aff_cpuset);
-#endif
-#ifdef __FreeBSD__
+#elif defined(__FreeBSD__)
 	/* Current thread */
 	if (0 == pid)
 		ret = cpuset_setaffinity(CPU_LEVEL_WHICH, CPU_WHICH_TID, -1,
@@ -82,8 +81,7 @@ set_affinity(pid_t pid)
 	ret = sched_getaffinity(pid, sizeof(cpumask), &cpumask);
 	if (ret != 0 || !CPU_EQUAL(&cpumask, &g_cfg.cpu_aff_cpuset))
 		return -1;
-#endif
-#ifdef __FreeBSD__
+#elif defined(__FreeBSD__)
 	if (pid == 0)
 		ret = cpuset_getaffinity(CPU_LEVEL_WHICH, CPU_WHICH_TID, -1,
 			sizeof(cpumask), &cpumask);
