@@ -1,7 +1,7 @@
 /*
  *   BSD LICENSE
  *
- *   Copyright(c) 2016-2018 Intel Corporation. All rights reserved.
+ *   Copyright(c) 2016-2019 Intel Corporation. All rights reserved.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -47,11 +47,11 @@
 #include "rdt.h"
 #include "cpu.h"
 
-static const struct pqos_cap *m_cap;
-static const struct pqos_cpuinfo *m_cpu;
-static const struct pqos_capability *m_cap_l2ca;
-static const struct pqos_capability *m_cap_l3ca;
-static const struct pqos_capability *m_cap_mba;
+static const struct pqos_cap *m_cap = NULL;
+static const struct pqos_cpuinfo *m_cpu = NULL;
+static const struct pqos_capability *m_cap_l2ca = NULL;
+static const struct pqos_capability *m_cap_l3ca = NULL;
+static const struct pqos_capability *m_cap_mba = NULL;
 
 /**
  * @brief Prints L2, L3 or MBA configuration in \a cfg
@@ -87,7 +87,7 @@ rdt_cfg_print(FILE *stream, const struct rdt_cfg cfg)
 		break;
 
 	case PQOS_CAP_TYPE_MBA:
-		fprintf(stream, "RATE: %u", cfg.u.mba->mb_rate);
+		fprintf(stream, "RATE: %u", cfg.u.mba->mb_max);
 		break;
 
 	default:
@@ -159,8 +159,9 @@ rdt_cfg_is_valid(const struct rdt_cfg cfg)
 			(cfg.u.l3->cdp == 0 && cfg.u.l3->u.ways_mask != 0));
 
 	case PQOS_CAP_TYPE_MBA:
-		return cfg.u.mba != NULL && cfg.u.mba->mb_rate > 0 &&
-			cfg.u.mba->mb_rate <= 100;
+		return cfg.u.mba != NULL && cfg.u.mba->mb_max > 0 &&
+			((cfg.u.mba->ctrl == 0 && cfg.u.mba->mb_max <= 100) ||
+			cfg.u.mba->ctrl == 1);
 
 	default:
 		break;
@@ -227,27 +228,32 @@ is_contiguous(const char *cat_type, const uint64_t bitmask)
 /**
  * @brief Function to get the highest resource ID (socket/cluster)
  *
- * @param technology used to determine if res ID should be socket or cluster
+ * @param [in] technology used to determine if res ID should be socket or
+ *             cluster
+ * @param [out] max_res_id the highest possible resource ID
  *
- * @return highest resource ID
- * @retval positive on success
- * @retval 0 on error
+ * @return Result
+ * @retval 0 on success
+ * @retval negative on error (-errno)
  */
 static int
-get_max_res_id(unsigned technology)
+get_max_res_id(unsigned technology, unsigned *max_res_id)
 {
         unsigned num_ids, max = 0, *ids = NULL;
+	unsigned i;
 
         if (m_cpu == NULL || technology == 0)
-                return 0;
+                return -EFAULT;
 
         /* get number of L2IDs */
         if (technology & (1 << PQOS_CAP_TYPE_L2CA)) {
                 ids = pqos_cpu_get_l2ids(m_cpu, &num_ids);
                 if (ids == NULL)
-                        return 0;
+                        return -EFAULT;
+
+                for (i = 0; i < num_ids; i++)
+                        max = ids[i] > max ? ids[i] : max;
 
-                max = num_ids > max ? num_ids : max;
                 free(ids);
         }
         /* get number of sockets */
@@ -255,12 +261,16 @@ get_max_res_id(unsigned technology)
             technology & (1 << PQOS_CAP_TYPE_MBA)) {
                 ids = pqos_cpu_get_sockets(m_cpu, &num_ids);
                 if (ids == NULL)
-                        return 0;
+                        return -EFAULT;
+
+                for (i = 0; i < num_ids; i++)
+                        max = ids[i] > max ? ids[i] : max;
 
-                max = num_ids > max ? num_ids : max;
                 free(ids);
         }
-        return max;
+
+        *max_res_id = max;
+        return 0;
 }
 
 /**
@@ -353,8 +363,11 @@ parse_mask_set(const char *cbm, const int force_dual_mask, uint64_t *mask,
 int
 parse_reset(const char *cpustr)
 {
-	unsigned cpustr_len = strlen(cpustr);
 	int ret = 0;
+	const unsigned cpustr_len = strnlen(cpustr, MAX_OPTARG_LEN);
+
+	if (cpustr_len == MAX_OPTARG_LEN)
+		return -EINVAL;
 
 	ret = str_to_cpuset(cpustr, cpustr_len, &g_cfg.reset_cpuset);
 	return ret > 0 ? 0 : ret;
@@ -434,7 +447,38 @@ rdt_mba_str_to_rate(const char *param, struct rdt_cfg mba)
 	if (ret < 0 || rate == 0 || rate > 100)
 		return -EINVAL;
 
-	mba.u.mba->mb_rate = rate;
+	mba.u.mba->ctrl = 0;
+	mba.u.mba->mb_max = rate;
+
+	return 0;
+}
+
+/**
+ * @brief Parses mbps string \a param and stores in \a mba
+ *
+ * @param [in] param mbps string
+ * @param [out] mba to store result
+ *
+ * @return status
+ * @retval 0 on success
+ * @retval negative on error (-errno)
+ */
+static int
+rdt_mba_str_to_mbps(const char *param, struct rdt_cfg mba)
+{
+	uint64_t mbps;
+	int ret;
+
+	if (PQOS_CAP_TYPE_MBA != mba.type || NULL == mba.u.generic_ptr ||
+			NULL == param)
+		return -EINVAL;
+
+	ret = str_to_uint64(param, 10, &mbps);
+	if (ret < 0 || mbps == 0)
+		return -EINVAL;
+
+	mba.u.mba->ctrl = 1;
+	mba.u.mba->mb_max = mbps;
 
 	return 0;
 }
@@ -558,15 +602,12 @@ parse_rdt(char *rdtstr)
 			break;
 
 		case 'b':
-			if (g_cfg.config[idx].mba_max > 0)
+			if (rdt_cfg_is_valid(mba))
 				return -EINVAL;
 
-			ret = str_to_uint64(param, 10,
-				&g_cfg.config[idx].mba_max);
-
-			if (ret != (int)strlen(param) ||
-					g_cfg.config[idx].mba_max == 0)
-				return -EINVAL;
+			ret = rdt_mba_str_to_mbps(param, mba);
+			if (ret < 0)
+				return ret;
 			break;
 
 		default:
@@ -581,16 +622,6 @@ parse_rdt(char *rdtstr)
 	if (CPU_COUNT(&g_cfg.config[idx].cpumask) == 0)
 		g_cfg.config[idx].pid_cfg = 1;
 
-	/* if software controller is used ...*/
-	if (g_cfg.config[idx].mba_max > 0) {
-		/* check if "core=" is configured */
-		if (!CPU_COUNT(&g_cfg.config[idx].cpumask))
-			return -EINVAL;
-		/* if initial MBA value is not set, set default (100%) */
-		if (!rdt_cfg_is_valid(mba))
-			g_cfg.config[idx].mba.mb_rate = MBA_SC_DEF_INIT_MBA;
-	}
-
 	if (!(rdt_cfg_is_valid(l2ca) || rdt_cfg_is_valid(l3ca) ||
 			rdt_cfg_is_valid(mba)))
 		return -EINVAL;
@@ -693,34 +724,120 @@ check_cpus(void)
 }
 
 /**
- * @brief Checks if CPU supports requested CDP configuration
+ * @brief Checks if CPU supports requested L3 CDP configuration
  *
  * @return status
  * @retval 0 on success
  * @retval negative on error (-errno)
  */
 static int
-check_cdp_support(void)
+check_l3cdp_support(void)
 {
 	unsigned i = 0;
-	const int cdp_supported = m_cap_l3ca != NULL &&
-		m_cap_l3ca->u.l3ca->cdp == 1;
-	const int cdp_enabled = cdp_supported &&
-		m_cap_l3ca->u.l3ca->cdp_on == 1;
+	int ret;
+	int l3cdp_supported = 0;
+	int l3cdp_enabled = 0;
 
-	if (cdp_enabled)
+	ret = pqos_l3ca_cdp_enabled(m_cap, &l3cdp_supported, &l3cdp_enabled);
+	if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
+		return -1;
+
+	if (l3cdp_enabled)
 		return 0;
 
 	for (i = 0; i < g_cfg.config_count; i++) {
 		if (0 == g_cfg.config[i].l3.cdp)
 			continue;
 
-		if (!cdp_supported)
-			fprintf(stderr, "Allocation: CDP requested but not "
+		if (!l3cdp_supported)
+			fprintf(stderr, "Allocation: L3 CDP requested but not "
 				"supported.\n");
 		else
-			fprintf(stderr, "Allocation: CDP requested but not "
-				"enabled. Please enable CDP.\n");
+			fprintf(stderr, "Allocation: L3 CDP requested but not "
+				"enabled. Please enable L3 CDP.\n");
+
+		return -ENOTSUP;
+	}
+
+	return 0;
+}
+
+/**
+ * @brief Checks if CPU supports requested L2 CDP configuration
+ *
+ * @return status
+ * @retval 0 on success
+ * @retval negative on error (-errno)
+ */
+static int
+check_l2cdp_support(void)
+{
+	unsigned i = 0;
+	int ret;
+
+	int l2cdp_supported = 0;
+	int l2cdp_enabled = 0;
+
+	ret = pqos_l2ca_cdp_enabled(m_cap, &l2cdp_supported, &l2cdp_enabled);
+	if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
+		return -1;
+
+	if (l2cdp_enabled)
+		return 0;
+
+	for (i = 0; i < g_cfg.config_count; i++) {
+		if (0 == g_cfg.config[i].l2.cdp)
+			continue;
+
+		if (!l2cdp_supported)
+			fprintf(stderr, "Allocation: L2 CDP requested but not "
+				"supported.\n");
+		else
+			fprintf(stderr, "Allocation: L2 CDP requested but not "
+				"enabled. Please enable L2 CDP.\n");
+
+		return -ENOTSUP;
+	}
+
+	return 0;
+}
+
+/**
+ * @brief Checks if CPU supports requested MBA CTRL configuration
+ *
+ * @return status
+ * @retval 0 on success
+ * @retval negative on error (-errno)
+ */
+static int
+check_mba_ctrl_support(void)
+{
+	unsigned i = 0;
+	int ret;
+	int ctrl_supported = 0;
+	int ctrl_enabled = 0;
+
+	/* MBA Software controller is available */
+	if (g_cfg.interface == PQOS_INTER_MSR)
+		return 0;
+
+	ret = pqos_mba_ctrl_enabled(m_cap, &ctrl_supported, &ctrl_enabled);
+	if (ret != PQOS_RETVAL_OK && ret != PQOS_RETVAL_RESOURCE)
+		return -1;
+
+	if (ctrl_enabled != 0)
+		return 0;
+
+	for (i = 0; i < g_cfg.config_count; i++) {
+		if (0 == g_cfg.config[i].mba.ctrl)
+			continue;
+
+		if (ctrl_supported == 0)
+			fprintf(stderr, "Allocation: MBA CTRL requested but "
+			        "not supported.\n");
+		else
+			fprintf(stderr, "Allocation: MBA CTRL requested but "
+				"not enabled. Please enable MBA CTRL.\n");
 
 		return -ENOTSUP;
 	}
@@ -756,8 +873,7 @@ check_supported(void)
 			return -ENOTSUP;
 		}
 
-		if ((rdt_cfg_is_valid(wrap_mba(&g_cfg.config[i].mba)) ||
-		     g_cfg.config[i].mba_max > 0) &&
+		if (rdt_cfg_is_valid(wrap_mba(&g_cfg.config[i].mba)) &&
                     NULL == m_cap_mba) {
 			fprintf(stderr, "Allocation: MBA requested but not "
 				"supported by system!\n");
@@ -949,7 +1065,15 @@ alloc_validate(void)
 	if (ret != 0)
 		return ret;
 
-	ret = check_cdp_support();
+	ret = check_l3cdp_support();
+	if (ret != 0)
+		return ret;
+
+	ret = check_l2cdp_support();
+	if (ret != 0)
+		return ret;
+
+	ret = check_mba_ctrl_support();
 	if (ret != 0)
 		return ret;
 
@@ -1136,7 +1260,11 @@ alloc_get_default_cos(struct pqos_l2ca *l2_def, struct pqos_l3ca *l3_def,
 
 	if (m_cap_mba != NULL && mba_def != NULL) {
 		memset(mba_def, 0, sizeof(*mba_def));
-		mba_def->mb_rate = 100;
+		if (m_cap_mba->u.mba->ctrl_on == 1) {
+			mba_def->ctrl = 1;
+			mba_def->mb_max = UINT32_MAX;
+		} else
+			mba_def->mb_max = 100;
 	}
 
 	return 0;
@@ -1166,10 +1294,10 @@ cfg_configure_cos(const struct pqos_l2ca *l2ca, const struct pqos_l3ca *l3ca,
 	const struct pqos_coreinfo *ci = NULL;
 	int ret;
 
-	if (NULL == l2ca || NULL == l3ca)
+	if (NULL == l2ca || NULL == l3ca || NULL == mba)
 		return -EINVAL;
 
-	if (NULL == m_cap_l2ca && NULL == m_cap_l3ca)
+	if (NULL == m_cap_l2ca && NULL == m_cap_l3ca && NULL == m_cap_mba)
 		return -EFAULT;
 
 
@@ -1233,6 +1361,10 @@ cfg_configure_cos(const struct pqos_l2ca *l2ca, const struct pqos_l3ca *l3ca,
 		/* if COS is not configured, set it to default */
 		if (!rdt_cfg_is_valid(wrap_mba(&mba_requested)))
 			mba_requested = mba_defs;
+		else if (mba_sc_mode(&g_cfg)) {
+			mba_requested.mb_max = MBA_SC_DEF_INIT_MBA;
+			mba_requested.ctrl = 0;
+		}
 
 		/* set proper COS id */
 		mba_requested.class_id = cos_id;
@@ -1294,12 +1426,12 @@ cfg_set_cores_os(const unsigned technology, const cpu_set_t *cores,
                 return -EFAULT;
         }
 
-        max_id = get_max_res_id(technology);
-        if (!max_id)
-                return -EFAULT;
+        ret = get_max_res_id(technology, &max_id);
+        if (ret != 0)
+                return ret;
 
         /* Set COS definition on necessary sockets/clusters */
-        for (i = 0; i < max_id; i++) {
+        for (i = 0; i <= max_id; i++) {
                 memset(core_array, 0, sizeof(core_array));
 
                 /* Get cores on res id i */
@@ -1344,12 +1476,12 @@ cfg_set_cores_msr(const unsigned technology, const cpu_set_t *cores,
 	int ret;
         unsigned i, max_id;
 
-        max_id = get_max_res_id(technology);
-        if (!max_id)
-                return -EFAULT;
+        ret = get_max_res_id(technology, &max_id);
+        if (ret != 0)
+                return ret;
 
         /* Assign new COS for all applicable res ids */
-        for (i = 0; i < max_id; i++) {
+        for (i = 0; i <= max_id; i++) {
                 unsigned core_array[CPU_SETSIZE] = {0};
                 unsigned core_num, cos_id;
 
@@ -1404,7 +1536,7 @@ cfg_set_pids(const unsigned technology, const struct pqos_l3ca *l3ca,
              const struct pqos_l2ca *l2ca, const struct pqos_mba *mba)
 {
         int ret = 0;
-	unsigned i, cos_id, max_id, num_pids = 1;
+	unsigned i, cos_id, num_pids = 1;
         pid_t pid = getpid(), *p = &pid;
 
         /* Assign COS to selected pids otherwise assign to this pid */
@@ -1427,26 +1559,51 @@ cfg_set_pids(const unsigned technology, const struct pqos_l3ca *l3ca,
                 goto set_pids_exit;
         }
 
-        max_id = get_max_res_id(technology);
-        if (!max_id)
-                return -EFAULT;
+        /* Set COS definitions across all res L2 IDs */
+        if (technology & (1 << PQOS_CAP_TYPE_L2CA)) {
+                unsigned num_ids, *ids;
 
-        /* Set COS definitions across all res IDs */
-        for (i = 0; i < max_id; i++) {
-                unsigned core;
+                ids = pqos_cpu_get_l2ids(m_cpu, &num_ids);
+                if (ids == NULL)
+                        return -EFAULT;
 
-                /* Get cores on res ID i */
-                if (technology & (1 << PQOS_CAP_TYPE_L2CA))
-                        ret = pqos_cpu_get_one_by_l2id(m_cpu, i, &core);
-                else
-                        ret = pqos_cpu_get_one_core(m_cpu, i, &core);
-                if (ret != 0)
-                        break;
+                for (i = 0; i < num_ids; i++) {
+                        unsigned core;
 
-                /* Configure COS on res ID i */
-                ret = cfg_configure_cos(l2ca, l3ca, mba, core, cos_id);
-                if (ret != 0)
-                        break;
+                        ret = pqos_cpu_get_one_by_l2id(m_cpu, ids[i], &core);
+                        if (ret != 0)
+                                break;
+
+                        /* Configure COS on res L2 ID i */
+                        ret = cfg_configure_cos(l2ca, l3ca, mba, core, cos_id);
+                        if (ret != 0)
+                                break;
+                }
+                free(ids);
+        }
+
+        /* Set COS definitions across all res L3 IDs */
+        if (technology & (1 << PQOS_CAP_TYPE_L3CA) ||
+            technology & (1 << PQOS_CAP_TYPE_MBA)) {
+                unsigned num_ids, *ids;
+
+                ids = pqos_cpu_get_sockets(m_cpu, &num_ids);
+                if (ids == NULL)
+                        return -EFAULT;
+
+                for (i = 0; i < num_ids; i++) {
+                        unsigned core;
+
+                        ret = pqos_cpu_get_one_core(m_cpu, ids[i], &core);
+                        if (ret != 0)
+                                break;
+
+                        /* Configure COS on res L3 ID i */
+                        ret = cfg_configure_cos(l2ca, l3ca, mba, core, cos_id);
+                        if (ret != 0)
+                                break;
+                }
+                free(ids);
         }
 set_pids_exit:
         if (ret != 0)
@@ -1690,3 +1847,13 @@ print_cmd_line_rdt_config(void)
 		}
 	}
 }
+
+void
+print_lib_version(void)
+{
+        int major = m_cap->version / 10000;
+        int minor = (m_cap->version % 10000) / 100;
+        int patch = m_cap->version % 100;
+
+        printf("PQoS Library version: %d.%d.%d\n", major, minor, patch);
+}
