@@ -1,7 +1,7 @@
 /*
  *   BSD LICENSE
  *
- *   Copyright(c) 2016-2018 Intel Corporation. All rights reserved.
+ *   Copyright(c) 2016-2019 Intel Corporation. All rights reserved.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -64,6 +64,9 @@ sudo_drop(void)
 	const char *sudo_uid = getenv("SUDO_UID");
 	const char *sudo_gid = getenv("SUDO_GID");
 	const char *sudo_user = getenv("SUDO_USER");
+	uid_t uid;
+	gid_t gid;
+	char *tailp;
 
 	/* Was sudo used to elevate privileges? */
 	if (NULL == sudo_uid || NULL == sudo_gid || NULL == sudo_user)
@@ -71,18 +74,15 @@ sudo_drop(void)
 
 	/* get user UID and GID */
 	errno = 0;
-	char *tailp = NULL;
-
-	const uid_t uid = (uid_t) strtol(sudo_uid, &tailp, 10);
-
+	tailp = NULL;
+	uid = (uid_t) strtol(sudo_uid, &tailp, 10);
 	if (NULL == tailp || *tailp != '\0' || errno != 0 ||
 			sudo_uid == tailp || 0 == uid)
 		goto err;
 
 	errno = 0;
 	tailp = NULL;
-	const gid_t gid = (gid_t) strtol(sudo_gid, &tailp, 10);
-
+	gid = (gid_t) strtol(sudo_gid, &tailp, 10);
 	if (NULL == tailp || *tailp != '\0' || errno != 0 ||
 			sudo_gid == tailp || 0 == gid)
 		goto err;
@@ -121,12 +121,12 @@ err:
 static int
 execute_cmd(int argc, char **argv)
 {
-	int i = 0;
-
 	if (0 >= argc || NULL == argv)
 		return -1;
 
 	if (g_cfg.verbose) {
+		int i;
+
 		printf("Trying to execute ");
 		for (i = 0; i < argc; i++)
 			printf("%s ", argv[i]);
@@ -223,7 +223,10 @@ print_usage(char *prgname, unsigned short_usage)
 	        "If not set the default implementation"
 	        " is to program the MSR's directly\n"
 		" -h, --help                            "
-		"display help\n\n");
+		"display help\n"
+		" -w, --version                         "
+		"display PQoS library version\n\n");
+
 
 	if (short_usage) {
 		printf("For more help run with -h/--help\n");
@@ -310,6 +313,7 @@ print_usage(char *prgname, unsigned short_usage)
  * @param [in] f_p flag for -p argument
  * @param [in] f_i flag for -I argument
  * @param [in] cmd flag for command to be executed
+ * @param [in] f_w flag for -w argument
  *
  * @return Operation status
  * @retval 1 on success
@@ -317,27 +321,20 @@ print_usage(char *prgname, unsigned short_usage)
  */
 static int
 validate_args(const int f_r, __attribute__((unused)) const int f_t,
-              const int f_c, const int f_p, const int f_i, const int cmd)
+              const int f_c, const int f_p, const int f_i, const int cmd,
+              const int f_w)
 {
         unsigned i;
         int f_n = 0; /**< non cpu (pid) config flag */
-        int f_sc = 0; /**< mba_max config flag */
 
         for (i = 0; i < g_cfg.config_count; i++) {
                 if (g_cfg.config[i].pid_cfg)
                         f_n++;
-                if (g_cfg.config[i].mba_max > 0)
-                        f_sc++;
                 /* Validate that only 1 pid config selected */
                 if (f_n > 1) {
                         fprintf(stderr, "Only 1 PID config allowed!\n");
                         return 0;
                 }
-                /* Validate that only 1 MBA SC configured */
-                if (f_sc > 1) {
-                        fprintf(stderr, "Only 1 MBA SC config allowed!\n");
-                        return 0;
-                }
         }
 
 	return (f_c && !f_p && cmd && !f_n) ||
@@ -345,8 +342,7 @@ validate_args(const int f_r, __attribute__((unused)) const int f_t,
 		(f_r && f_p && !cmd) ||
 		(f_i && f_n && !f_p && cmd) ||
 		(f_i && f_n && f_p && !cmd) ||
-		(!f_i && f_sc && !f_p && cmd) ||
-		(!f_i && f_sc && f_p && !cmd);
+		f_w;
 }
 
 /**
@@ -410,10 +406,11 @@ parse_args(int argc, char **argv)
 		{ "verbose",	no_argument,		0, 'v' },
 		{ "iface-os",   no_argument,            0, 'I' },
 		{ "help",	no_argument,		0, 'h' },
+		{ "version",	no_argument,		0, 'w' },
 		{ NULL, 0, 0, 0 } };
 
 	while ((opt = getopt_long(argc, argvopt,
-                                  "+c:p:r:t:kvIh", lgopts, NULL)) != -1) {
+	                          "+c:p:r:t:kvIhw", lgopts, NULL)) != -1) {
 		switch (opt) {
 		case 'c':
 			retval = parse_cpu(optarg);
@@ -423,7 +420,7 @@ parse_args(int argc, char **argv)
 			}
 			break;
 		case 'p':
-                        retval = parse_pids(optarg);
+			retval = parse_pids(optarg);
 			if (retval != 0) {
 				fprintf(stderr, "Invalid PID parameters!\n");
 				goto exit;
@@ -458,6 +455,9 @@ parse_args(int argc, char **argv)
 		case 'h':
 			retval = -EAGAIN;
 			goto exit;
+		case 'w':
+			g_cfg.show_version = 1;
+			break;
 		}
 	}
 
@@ -529,7 +529,7 @@ rdtset_init(void)
         }
 
         /* Initialize MBA SW controller */
-        if (mba_sc_mode()) {
+        if (mba_sc_mode(&g_cfg)) {
                 ret = mba_sc_init();
                 if (ret < 0) {
                         fprintf(stderr, "%s,%s:%d MBA SC init failed!\n",
@@ -596,7 +596,8 @@ main(int argc, char **argv)
 			0 != CPU_COUNT(&g_cfg.cpu_aff_cpuset),
 			0 != g_cfg.pid_count,
                         0 != g_cfg.interface,
-			0 != g_cfg.command)) {
+			0 != g_cfg.command,
+			0 != g_cfg.show_version)) {
 		fprintf(stderr, "Incorrect invocation!\n");
 		print_usage(argv[0], 1);
 		exit(EXIT_FAILURE);
@@ -612,6 +613,12 @@ main(int argc, char **argv)
         if (ret < 0)
                 exit(EXIT_FAILURE);
 
+	/* display library version */
+	if (0 != g_cfg.show_version) {
+		print_lib_version();
+		exit(EXIT_SUCCESS);
+	}
+
 	/* reset COS association */
 	if (0 != CPU_COUNT(&g_cfg.reset_cpuset)) {
 		if (g_cfg.verbose)
@@ -666,7 +673,7 @@ main(int argc, char **argv)
                         }
         }
 
-        if (mba_sc_mode()) {
+        if (mba_sc_mode(&g_cfg)) {
                 mba_sc_main(child);
 
         } else if (0 != g_cfg.command) {
@@ -677,7 +684,7 @@ main(int argc, char **argv)
                         exit(EXIT_FAILURE);
         }
 
-	if (0 != g_cfg.command || mba_sc_mode())
+	if (0 != g_cfg.command || mba_sc_mode(&g_cfg))
 		/*
 		 * If we were running some command or doing MBA SW control,
 		 * do clean-up. Clean-up function is executed on process exit.
